
from django.core.exceptions import ValidationError
from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ..text.models import CharFieldDefinition


class EmailFieldDefinition(CharFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'email field')
        verbose_name_plural = _(u'email fields')
        defined_field_class = fields.EmailField
        defined_field_category = _(u'web')
        
class URLFieldDefinition(CharFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'URL field')
        verbose_name_plural = _(u'URL fields')
        defined_field_class = fields.URLField
        defined_field_category = _(u'web')
        
class SlugFieldDefinition(CharFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'slug field')
        verbose_name_plural = _(u'slug fields')
        defined_field_class = fields.SlugField
        defined_field_category = _(u'web')
        
class IPAddressFieldDefinition(CharFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'IP address field')
        verbose_name_plural = _(u'IP address fields')
        defined_field_class = fields.IPAddressField
        defined_field_category = _(u'web')

# We should eat our own dogfood and
# provide those options as FieldDefinitionOptionChoice initial_data
# fixture
try:
    # Django 1.4+
    GenericIPAddressField = fields.GenericIPAddressField
except AttributeError:
    pass
else:
    PROTOCOL_CHOICES = (('both', _(u'both')),
                        ('IPv4', _(u'IPv4')),
                        ('IPv6', _(u'IPv6')))
    
    protocol_help_text = _(u'Limits valid inputs to the specified protocol.')
    
    unpack_ipv4_help_text = _(u'Unpacks IPv4 mapped addresses like '
                              u'``::ffff::192.0.2.1`` to ``192.0.2.1``')
    
    class GenericIPAddressFieldDefinition(CharFieldDefinition):
        
        protocol = fields.CharField(_(u'protocol'), max_length=4,
                                    choices=PROTOCOL_CHOICES, default='both')
        
        unpack_ipv4 = fields.BooleanField(_(u'unpack ipv4'), default=False)
        
        class Meta:
            app_label = 'mutant'
            verbose_name = _(u'generic IP address field')
            verbose_name_plural = _(u'generic IP address fields')
            defined_field_class = GenericIPAddressField
            defined_field_options = ('protocol', 'unpack_ipv4',)
            defined_field_category = _(u'web')
            
        def clean(self):
            if self.unpack_ipv4 and self.procotol != 'both':
                msg = _(u"Can only be used when ``protocol`` is set to 'both'.")
                raise ValidationError({'unpack_ipv4': msg})




from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _
from djangotoolbox import fields
from picklefield.fields import PickledObjectField

from ....hacks import patch_db_field_compare
from ....models import FieldDefinition


# Since we use a pickled field that might contain a Field instance
# we must make sure their cmp methods works correctly.
patch_db_field_compare()

class AbstractIterableFieldDefinition(FieldDefinition):
    
    item_field = PickledObjectField(_(u'item field'), blank=True, null=True)
    
    class Meta:
        abstract = True
        defined_field_options = ('item_field',)
        defined_field_category = _(u'nonrel')

class DictFieldDefinition(AbstractIterableFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'dict field')
        verbose_name_plural = _(u'dict fields')
        defined_field_class = fields.DictField

class ListFieldDefinition(AbstractIterableFieldDefinition):
    
    ordering = PickledObjectField(_(u'ordering'), blank=True, null=True)
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'list field')
        verbose_name_plural = _(u'list fields')
        defined_field_class = fields.ListField
        defined_field_options = ('ordering',)

# This class isn't working correctly on pymongo
# It doesn't know how to encode set... see testcases
# InvalidDocument: Cannot encode object: set(['A', 'Y', 'C', 'M'])
class SetFieldDefinition(AbstractIterableFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'set field')
        verbose_name_plural = _(u'set fields')
        defined_field_class = fields.SetField
        
class BlobFieldDefinition(FieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'blob field')
        verbose_name_plural = _(u'blob fields')
        defined_field_class = fields.BlobField
        defined_field_category = _(u'nonrel')
        
class EmbeddedModelFieldDefinition(FieldDefinition):
    
    model = models.ForeignKey(ContentType, verbose_name=_(u'model'),
                              related_name='+', blank=True, null=True)
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'embedded model field')
        verbose_name_plural = _(u'embedded model fields')
        defined_field_class = fields.EmbeddedModelField
        defined_field_category = _(u'nonrel')
    
    def get_field_options(self):
        options = super(EmbeddedModelFieldDefinition, self).get_field_options()
        if self.model:
            options['model'] = self.model.model_class()
        return options
    



from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ...models import FieldDefinition

auto_now_help_text = _(u'Automatically set the field to now every time the '
                       u'object is saved.')

auto_now_add_help_text = _(u'Automatically set the field to now when the '
                           u'object is first created.')

class DateFieldDefinition(FieldDefinition):
    
    auto_now = fields.BooleanField(_(u'auto now'), default=False,
                                   help_text=auto_now_help_text)
    
    auto_now_add = fields.BooleanField(_(u'auto_now_add'), default=False,
                                       help_text=auto_now_add_help_text)
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'date field')
        verbose_name_plural = _(u'date fields')
        defined_field_class = fields.DateField
        defined_field_options = ('auto_now', 'auto_now_add',)
        defined_field_category = _(u'datetime')

class TimeFieldDefinition(DateFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'time field')
        verbose_name_plural = _(u'time fields')
        defined_field_class = fields.TimeField

class DateTimeFieldDefinition(DateFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'date time field')
        verbose_name_plural = _(u'date time fields')
        defined_field_class = fields.DateTimeField




from django.db.models.signals import post_delete, post_save, pre_delete

from ....management import allow_syncdbs, perform_ddl

from ..models import ManyToManyFieldDefinition


def many_to_many_field_definition_post_save(sender, instance, created, **kwargs):
    """
    This is not connected atm since there's an issue while used as a signal
    """
    if created:
        model_class = instance.model_def.model_class(force_create=True)
        field = model_class._meta.get_field(str(instance.name))
        options = field.rel.through._meta
        intermediary_table_name = options.db_table
        intermediary_table_fields = tuple((field.name, field)
                                          for field in options.fields)
        perform_ddl(model_class, 'create_table',
                    intermediary_table_name, intermediary_table_fields)
    else:
        #TODO: track field and model rename in order to rename the intermediaray
        # table...
        pass
    
#post_save.connect(many_to_many_field_definition_post_save, ManyToManyFieldDefinition,
#                  dispatch_uid='mutant.contrib.related.management.many_to_many_field_definition_post_save')

def many_to_many_field_definition_pre_delete(sender, instance, **kwargs):
    model_class = instance.model_def.model_class()
    field = model_class._meta.get_field(str(instance.name))
    intermediary_table_name = field.rel.through._meta.db_table
    instance._state._m2m_deletion = (
        allow_syncdbs(model_class),
        intermediary_table_name
    )

pre_delete.connect(many_to_many_field_definition_pre_delete, ManyToManyFieldDefinition,
                   dispatch_uid='mutant.contrib.related.management.many_to_many_field_definition_pre_delete')

def many_to_many_field_definition_post_delete(sender, instance, **kwargs):
    syncdbs, intermediary_table_name = instance._state._m2m_deletion
    for db in syncdbs:
        db.delete_table(intermediary_table_name)
    del instance._state._m2m_deletion

post_delete.connect(many_to_many_field_definition_post_delete, ManyToManyFieldDefinition,
                    dispatch_uid='mutant.contrib.related.management.many_to_many_field_definition_post_delete')




from django.contrib.gis.geos import (GeometryCollection, LineString, Point,
    Polygon, MultiLineString, MultiPoint, MultiPolygon)

from ...models import BaseDefinition
from ...test.testcases import FieldDefinitionTestMixin
from ...tests.models import BaseModelDefinitionTestCase

from .models import (GeoModel, GeometryCollectionFieldDefinition,
    LineStringFieldDefinition, PointFieldDefinition, PolygonFieldDefinition,
    MultiLineStringFieldDefinition, MultiPointFieldDefinition,
    MultiPolygonFieldDefinition)


class GeometryFieldDefinitionBaseTest(BaseModelDefinitionTestCase):
    
    def setUp(self):
        super(GeometryFieldDefinitionBaseTest, self).setUp()
        BaseDefinition.objects.create(model_def=self.model_def, base=GeoModel)

class PointFieldDefinitionTest(FieldDefinitionTestMixin,
                               GeometryFieldDefinitionBaseTest):
    field_definition_cls = PointFieldDefinition
    field_values = (Point(5, 23), Point(13, 37))
    
class LineStringFieldDefinitionTest(FieldDefinitionTestMixin,
                                    GeometryFieldDefinitionBaseTest):
    field_definition_cls = LineStringFieldDefinition
    field_values = (
        LineString((0, 0), (0, 50), (50, 50), (50, 0), (0, 0)),
        LineString((1, 2), (3, 4), (5, 6), (7, 8), (9, 10))
    )

class PolygonFieldDefinitionTest(FieldDefinitionTestMixin,
                                 GeometryFieldDefinitionBaseTest):
    field_definition_cls = PolygonFieldDefinition
    field_values = (
        Polygon( ((0.0, 0.0), (0.0, 50.0), (50.0, 50.0), (50.0, 0.0), (0.0, 0.0)) ),
        Polygon( ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0)) ),
    )

class MultiLineStringFieldDefinitionTest(FieldDefinitionTestMixin,
                                         GeometryFieldDefinitionBaseTest):
    field_definition_cls = MultiLineStringFieldDefinition
    field_values = (
        MultiLineString(
            LineString((0, 0), (0, 50), (50, 50), (50, 0), (0, 0)),
            LineString((1, 2), (3, 4), (5, 6), (7, 8), (9, 10)),
        ),
        MultiLineString(
            LineString((13, 7), (18, 50), (50, 50), (50, 27), (110, 0)),
            LineString((1, 2), (3, 4), (5, 6), (7, 8), (9, 10)),
            LineString((0, 0), (0, 50), (50, 50), (50, 0), (0, 0)),
        ),
    )

class MultiPointFieldDefinitionTest(FieldDefinitionTestMixin,
                                    GeometryFieldDefinitionBaseTest):
    field_definition_cls = MultiPointFieldDefinition
    field_values = (
        MultiPoint(Point(0, 0), Point(1, 1)),
        MultiPoint(Point(5, 23), Point(13, 37), Point(13, 58)),
    )
    
class MultiPolygonFieldDefinitionTest(FieldDefinitionTestMixin,
                                      GeometryFieldDefinitionBaseTest):
    field_definition_cls = MultiPolygonFieldDefinition
    field_values = (
        MultiPolygon(
            Polygon( ((0.0, 0.0), (0.0, 50.0), (50.0, 50.0), (50.0, 0.0), (0.0, 0.0)) ),
            Polygon( ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0)) ),
        ),
        MultiPolygon(
            Polygon( ((0.0, 0.0), (0.0, 50.0), (50.0, 50.0), (50.0, 0.0), (0.0, 0.0)) ),
            Polygon( ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0)) ),
            Polygon( ((0.0, 0.0), (0.0, 50.0), (50.0, 51.0), (50.0, 45), (0.0, 0.0)) ),
        ),
    )

class GeometryCollectionFieldDefinitionTest(FieldDefinitionTestMixin,
                                            GeometryFieldDefinitionBaseTest):
    field_definition_cls = GeometryCollectionFieldDefinition
    field_values = (
        GeometryCollection(
            Point(0, 0),
            Polygon( ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0)) ),
        ),
        GeometryCollection(
            LineString((1, 2), (3, 4), (5, 6), (7, 8), (9, 10)),
            Polygon( ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0)) ),
            Point(5, 23),
            Point(13, 37),
        ),
    )
    



import re

import django
from django.contrib.contenttypes.models import ContentType
from django.db import connections
from django.db.utils import IntegrityError
from django.test.testcases import TestCase
from south.db import dbs as south_dbs

from ..models.model import ModelDefinition


def connections_support_ddl_transactions():
    """
    Returns True if all connections support ddl transactions
    """
    return all(south_dbs[name].has_ddl_transactions for name in connections)

class DDLTestCase(TestCase):
    """
    A class that behaves like TestCase if all connections support ddl
    transactions or like TransactionTestCase if it's not the case.
    """
    
    def _fixture_setup(self):
        if not connections_support_ddl_transactions():
            return super(TestCase, self)._fixture_setup()
        else:
            return super(DDLTestCase, self)._fixture_setup()
        
    def _fixture_teardown(self):
        if not connections_support_ddl_transactions():
            return super(TestCase, self)._fixture_teardown()
        else:
            return super(DDLTestCase, self)._fixture_teardown()
        
class ModelDefinitionDDLTestCase(DDLTestCase):
    
    def tearDown(self):
        if not connections_support_ddl_transactions():
            # We must delete the ModelDefinition tables by ourself since
            # they won't be removed by a rollback.
            for md in ModelDefinition.objects.all():
                md.delete()
        ContentType.objects.clear_cache()

class VersionCompatMixinTestCase(TestCase):
    
    # Django < 1.4 doesn't have assertIsIntance and `ordered` kwarg for assertQuerysetEqual
    if django.VERSION[0] == 1 and django.VERSION[1] < 4:
        def assertRaisesMessage(self, expected_exception, expected_message,
                                callable_obj=None, *args, **kwargs):
            return self.assertRaisesRegexp(expected_exception,
                    re.escape(expected_message), callable_obj, *args, **kwargs)
            
        def assertQuerysetEqual(self, qs, values, transform=repr, ordered=True):
            if not ordered:
                return self.assertEqual(set(map(transform, qs)), set(values))
            return self.assertEqual(map(transform, qs), values)
        
class FieldDefinitionTestMixin(object):
    
    field_defintion_init_kwargs = {}
    field_values = ()
    
    def setUp(self):
        super(FieldDefinitionTestMixin, self).setUp()
        self.field = self.field_definition_cls.objects.create(model_def=self.model_def,
                                                              name='field',
                                                              **self.field_defintion_init_kwargs)
    
    def get_field_value(self, instance, name='field'):
        return getattr(instance, name)
    
    def prepare_default_value(self, value):
        return value
    
    def test_field_default(self):
        default = self.prepare_default_value(self.field_values[0])
        field = self.field
        
        field.default = default
        field.full_clean()
        field.save()
        
        Model = self.model_def.model_class()
        instance = Model.objects.create()
        created_default = self.prepare_default_value(self.get_field_value(instance))
        self.assertEqual(created_default, default)
        
    def test_model_save(self):
        first_value, second_value = self.field_values
        
        Model = self.model_def.model_class()
        instance = Model.objects.create(field=first_value)
        self.assertEqual(self.get_field_value(instance), first_value)
        
        instance.field = second_value
        instance.save()
        instance = Model.objects.get()
        self.assertEqual(self.get_field_value(instance), second_value)
        
    def test_field_renaming(self):
        value = self.field_values[0]
        Model = self.model_def.model_class()
        
        Model.objects.create(field=value)
        
        self.field.name = 'renamed_field'
        self.field.save()
        
        instance = Model.objects.get()
        self.assertEqual(self.get_field_value(instance, 'renamed_field'), value)
        
        msg = "'field' is an invalid keyword argument for this function"
        self.assertRaisesMessage(TypeError, msg, Model, field=value)
        
        Model.objects.create(renamed_field=value)
        
    def test_field_deletion(self):
        value = self.field_values[0]
        Model = self.model_def.model_class()
        
        Model.objects.create(field=value)

        self.field.delete()
        
        msg = "'field' is an invalid keyword argument for this function"
        self.assertRaisesMessage(TypeError, msg, Model, field=value)
        
    def test_field_unique(self):
        value = self.field_values[0]
        Model = self.model_def.model_class()
        
        self.field.unique = True
        self.field.save()
        
        Model.objects.create(field=value)
        with self.assertRaises(IntegrityError):
            Model.objects.create(field=value)




from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import ugettext_lazy as _


class MutableModel(models.Model):
    """
    Abstract class used to identify models that we're created by a definition
    """
    class Meta:
        abstract = True
    
    @classmethod 
    def definition(cls):
        definition_cls, definition_pk = cls._definition
        return definition_cls.objects.get(pk=definition_pk)
            
    @classmethod
    def is_obsolete(cls):
        return cls._is_obsolete
    
    def clean(self):
        if self.is_obsolete():
            raise ValidationError('Obsolete definition')
        return super(MutableModel, self).clean()
    
    def save(self, *args, **kwargs):
        if self.is_obsolete():
            msg = _(u'Cannot save an obsolete model')
            raise ValidationError(msg)
        return super(MutableModel, self).save(*args, **kwargs)
    
    def delete(self, *args, **kwargs):
        if self.is_obsolete():
            msg = _(u'Cannot delete an obsolete model')
            raise ValidationError(msg)
        return super(MutableModel, self).delete(*args, **kwargs)




from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError

from ...test.testcases import FieldDefinitionTestMixin
from ...tests.models.utils import BaseModelDefinitionTestCase

from .models import (DictFieldDefinition, EmbeddedModelFieldDefinition,
    ListFieldDefinition, SetFieldDefinition)


class DictFieldDefinitionTest(FieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = DictFieldDefinition
    field_defintion_init_kwargs = {'default': {'1': 2}}
    field_values = (
        {'key': 'value', 'other': 1337},
        {'somewhere': None, 'I': 'Belong'}
    )

def identity(obj):
    return obj

class ListFieldDefinitionTest(FieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = ListFieldDefinition
    field_defintion_init_kwargs = {'default': ['Y', 'M', 'C', 'A']}
    field_values = (
        ['A', 'C', 'M', 'Y'],
        ['A', 'B', 'C', 1, 2, 3]
    )
    
    def test_field_ordering(self):
        field = self.field
        
        field.ordering = 4
        msg = "'ordering' has to be a callable or None, not of type <type 'int'>"
        self.assertRaisesMessage(ValidationError, msg, field.full_clean)
        
        field.ordering = identity
        field.full_clean()
        field.save()

class SetFieldDefinitionTest(FieldDefinitionTestMixin,
                             BaseModelDefinitionTestCase):
    field_definition_cls = SetFieldDefinition
    field_defintion_init_kwargs = {'default': set(['Y', 'M', 'C', 'A'])}
    field_values = (
        set(['A', 'B', 'C', 1, 2, 3]),
        set(['I', 'LOVE', 'ROCK', 'N', 'ROLL'])
    )

def default_embedded():
    return ContentType(app_label='abc', model='DoTheDance')

class EmbeddedModelFieldTest(FieldDefinitionTestMixin,
                             BaseModelDefinitionTestCase):
    field_definition_cls = EmbeddedModelFieldDefinition
    field_defintion_init_kwargs = {'default': default_embedded}
    field_values = (
        ContentType(app_label='pyt', model='The way you move is a mistery'),
        ContentType(app_label='everybody', model='Knows this is nowhere'),
    )
    
    def test_field_default(self):
        # TODO: Find out why this fails sometimes
        pass
    
    def test_field_unique(self):
        # TODO: Find out why this fails sometimes
        pass




from itertools import groupby
from operator import itemgetter

group_item_getter = itemgetter('group')
def choices_from_dict(choices):
    for grp, choices in groupby(choices, key=group_item_getter):
        if grp is None:
            for choice in choices:
                yield (choice['value'], choice['label'])
        else:
            yield (grp, tuple((choice['value'], choice['label'])
                                for choice in choices))




from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.core.management import call_command
from django.db import models, router
from django.db.utils import IntegrityError
from django.utils.translation import ugettext

from mutant.contrib.text.models import CharFieldDefinition
from mutant.contrib.related.models import ForeignKeyDefinition
from mutant.db.models import MutableModel
from mutant.models.model import (ModelDefinition, OrderingFieldDefinition,
    UniqueTogetherDefinition, BaseDefinition)
from mutant.tests.models.utils import (BaseModelDefinitionTestCase,
    skipUnlessMutantModelDBFeature)


try:
    raise ImportError()
    from test.test_support import captured_stderr
except ImportError:
    # python 2.6 doesn't provide this helper
    from contextlib import contextmanager
    import StringIO
    import sys

    @contextmanager
    def captured_stderr():
        stderr = sys.stderr
        try:
            sys.stderr = StringIO.StringIO()
            yield sys.stderr
        finally:
            sys.stderr = stderr

class ModelDefinitionManipulationTest(BaseModelDefinitionTestCase):
    
    def test_model_class_creation_cache(self):
        Model = self.model_def.model_class()
        self.assertEqual(Model, self.model_def.model_class())
        
        self.assertNotEqual(Model, self.model_def.model_class(force_create=True))

    def test_rename_model(self):
        """
        Make sure changing the app_label or object_name renames the associated
        table
        """
        def get_table_name():
            return self.model_def.model_class()._meta.db_table
        
        db = router.db_for_read(self.model_def.model_class())
        
        table_name = get_table_name()
        self.model_def.app_label = 'myapp'
        self.model_def.save()
        self.assertTableDoesntExists(db, table_name)
        table_name = get_table_name()
        self.assertTableExists(db, table_name)
        
        self.model_def.object_name = 'MyModel'
        self.model_def.save()
        self.assertTableDoesntExists(db, table_name)
        table_name = get_table_name()
        self.assertTableExists(db, table_name)
        
        self.model_def.delete()
        self.assertTableDoesntExists(db, table_name)
        
    def test_fixture_loading(self):
        call_command('loaddata', 'fixture_loading_test', verbosity=0, commit=False)
        model_def = ModelDefinition.objects.get(app_label='myfixtureapp',
                                                object_name='MyFixtureModel')
        Model = model_def.model_class()
        Model.objects.create()
        
    def test_verbose_name(self):
        Model = self.model_def.model_class()
        
        self.assertEqual(Model._meta.verbose_name, self.model_def.model)
        self.assertEqual(Model._meta.verbose_name_plural, "%ss" % self.model_def.model)
        
        self.model_def.verbose_name = u'MyMoDeL'
        self.model_def.verbose_name_plural = u'MyMoDeLZ0Rs'
        self.model_def.save()
        
        self.assertEqual(Model._meta.verbose_name, ugettext(u'MyMoDeL'))
        self.assertEqual(Model._meta.verbose_name_plural, ugettext(u'MyMoDeLZ0Rs'))
        
    def test_multiple_model_definition(self):
        """
        Make sure multiple model definition can coexists
        """
        other_model_def = ModelDefinition.objects.create(app_label='app',
                                                         object_name='OtherModel')
        
        self.assertNotEqual(other_model_def.model_class(),
                            self.model_def.model_class())
        
        self.assertNotEqual(other_model_def.model_ct, self.model_def.model_ct)
            
class ModelValidationTest(BaseModelDefinitionTestCase):
    
    def test_installed_app_override_failure(self):
        """
        Make sure we can't save a model definition with an app_label of
        an installed app.
        """
        self.model_def.app_label = 'mutant'
        self.assertRaises(ValidationError, self.model_def.clean)

class ModelClassProxyProxyTests(BaseModelDefinitionTestCase):
    
    def test_proxy_interactions(self):
        CharFieldDefinition.objects.create(model_def=self.model_def,
                                           name="name", max_length=10)
        Model = self.model_def.model_class()
        
        sergei = Model.objects.create(name='Sergei')
        
        halak = Model(name='Halak')
        halak.save()
        
        assert issubclass(Model, models.Model)
        
        assert issubclass(Model, MutableModel)
        
        assert unicode(Model) == u"<class 'mutant.apps.app.models.Model'>"
        
        assert sergei == Model.objects.get(name='Sergei')
        
        class A(object):
            
            class_model = Model
            
            def __init__(self, model):
                self.model = model
        
        a = A(Model)
        
        assert Model == a.model
        assert Model == A.class_model
        
        a.model = Model # Assign a proxy
        a.model = a.model # Assign a Model
        a.model = 4

    def test_definition_deletion(self):
        CharFieldDefinition.objects.create(model_def=self.model_def,
                                           name="name", max_length=10)
        
        Model = self.model_def.model_class()
        db = router.db_for_write(Model)
        instance = Model.objects.create(name="Quebec")
        table_name = Model._meta.db_table
        self.model_def.delete()
        self.assertTableDoesntExists(db, table_name)
        
        with self.assertRaises(AttributeError):
            Model(name="name")
        
        with self.assertRaises(AttributeError):
            Model.objects.all()
           
        with self.assertRaises(ValidationError):
            instance.clean()
            
        with self.assertRaises(ValidationError):
            instance.save()
        
        with self.assertRaises(ValidationError):
            instance.delete()
            
        self.setUp() # Recreate the model definition to delete it in tearDown

class OrderingDefinitionTest(BaseModelDefinitionTestCase):
    
    def setUp(self):
        super(OrderingDefinitionTest, self).setUp()
        self.f1 = CharFieldDefinition.objects.create(model_def=self.model_def,
                                                     name='f1', max_length=25)
        ct_ct = ContentType.objects.get_for_model(ContentType)
        self.f2 = ForeignKeyDefinition.objects.create(model_def=self.model_def,
                                                      null=True,
                                                      name='f2', to=ct_ct)

    def test_clean(self):
        ordering = OrderingFieldDefinition(model_def=self.model_def)
        
        ordering.lookup = '?'
        ordering.clean()
        
        ordering.lookup = 'f1'
        ordering.clean()
        
        ordering.lookup = 'f2__app_label'
        ordering.clean()
        
        with self.assertRaises(ValidationError):
            ordering.lookup = 'f3'
            ordering.clean()
            
        with self.assertRaises(ValidationError):
            ordering.lookup = 'f2__higgs_boson'
            ordering.clean()

    @skipUnlessMutantModelDBFeature('supports_joins')
    def test_simple_ordering(self):
        Model = self.model_def.model_class()
        model_ct = ContentType.objects.get_for_model(Model) #app
        ct_ct = ContentType.objects.get_for_model(ContentType) #contenttypes
        Model.objects.create(f1='Simon', f2=ct_ct)
        Model.objects.create(f1='Alexander', f2=model_ct)
        
        
        # Instances should be sorted by id
        self.assertQuerysetEqual(Model.objects.values('f1'),
                                 [u'Simon', u'Alexander'],
                                 transform=lambda x: x['f1'], ordered=True)
        
        # Instances should be sorted by f1 and not id
        f1_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                             lookup='f1')
        self.assertQuerysetEqual(Model.objects.values('f1'),
                                 [u'Alexander', u'Simon'],
                                 transform=lambda x: x['f1'], ordered=True)
        
        # Swap the ordering to descending
        f1_ordering.descending = True
        f1_ordering.save()
        self.assertQuerysetEqual(Model.objects.values('f1'),
                                 [u'Simon', u'Alexander'],
                                 transform=lambda x: x['f1'], ordered=True)
        f1_ordering.delete()
        
        # Order by f2__app_label
        f2_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                             lookup='f2__app_label')
        self.assertQuerysetEqual(Model.objects.values('f1'),
                                 [u'Alexander', u'Simon'],
                                 transform=lambda x: x['f1'], ordered=True)
        
        # Swap the ordering to descending
        f2_ordering.descending = True
        f2_ordering.save()
        self.assertQuerysetEqual(Model.objects.values('f1'),
                                 [u'Simon', u'Alexander'],
                                 transform=lambda x: x['f1'], ordered=True)
        f2_ordering.delete()
    
    @skipUnlessMutantModelDBFeature('supports_joins')
    def test_multiple_ordering(self):
        Model = self.model_def.model_class()
        model_ct = ContentType.objects.get_for_model(Model) #app
        ct_ct = ContentType.objects.get_for_model(ContentType) #contenttypes
        Model.objects.create(f1='Simon', f2=ct_ct)
        Model.objects.create(f1='Alexander', f2=model_ct)
        Model.objects.create(f1='Julia', f2=ct_ct)
        Model.objects.create(f1='Alexander', f2=ct_ct)
        
        f1_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                             lookup='f1')
        f2_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                             lookup='f2__app_label')
        
        
        self.assertQuerysetEqual(Model.objects.values('f1', 'f2__app_label'),
                                 [(u'Alexander', u'app'), (u'Alexander', u'contenttypes'),
                                  (u'Julia', u'contenttypes'), (u'Simon', u'contenttypes')],
                                 transform=lambda x: (x['f1'], x['f2__app_label']),
                                 ordered=True)
        
        # Swap the ordering to descending
        f2_ordering.descending = True
        f2_ordering.save()
        self.assertQuerysetEqual(Model.objects.values('f1', 'f2__app_label'),
                                 [(u'Alexander', u'contenttypes'), (u'Alexander', u'app'),
                                  (u'Julia', u'contenttypes'), (u'Simon', u'contenttypes')],
                                 transform=lambda x: (x['f1'], x['f2__app_label']),
                                 ordered=True)
        
        # Swap order
        f1_ordering.order, f2_ordering.order = f2_ordering.order, f1_ordering.order
        f1_ordering.save()
        f2_ordering.save()
        self.assertQuerysetEqual(Model.objects.values('f1', 'f2__app_label'),
                                 [(u'Alexander', u'contenttypes'), (u'Julia', u'contenttypes'),
                                  (u'Simon', u'contenttypes'), (u'Alexander', u'app')],
                                 transform=lambda x: (x['f1'], x['f2__app_label']),
                                 ordered=True)
        
        # Swap the ordering to descending
        f1_ordering.descending = True
        f1_ordering.save()
        self.assertQuerysetEqual(Model.objects.values('f1', 'f2__app_label'),
                                 [(u'Simon', u'contenttypes'), (u'Julia', u'contenttypes'),
                                  (u'Alexander', u'contenttypes'), (u'Alexander', u'app')],
                                 transform=lambda x: (x['f1'], x['f2__app_label']),
                                 ordered=True)
        
        f1_ordering.delete()
        f2_ordering.delete()

class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
    
    def setUp(self):
        super(UniqueTogetherDefinitionTest, self).setUp()
        self.f1 = CharFieldDefinition.objects.create(model_def=self.model_def,
                                                     name='f1', max_length=25)
        self.f2 = CharFieldDefinition.objects.create(model_def=self.model_def,
                                                     name='f2', max_length=25)
        self.ut = UniqueTogetherDefinition.objects.create(model_def=self.model_def)
        self.Model = self.model_def.model_class()

    def test_clean(self):
        """
        Make sure we can't create a unique key with two fields of two
        different models
        """
        other_model_def = ModelDefinition.objects.create(app_label='app',
                                                         object_name='OtherModel')
        
        f2 = CharFieldDefinition.objects.create(model_def=other_model_def,
                                                name='f2', max_length=25)
        self.ut.field_defs.add(self.f1, f2)
        with self.assertRaises(ValidationError):
            self.ut.clean()
        other_model_def.delete()

    def test_cannot_create_unique(self):
        """
        Creating a unique key on a table with duplicate 
        rows shouldn't work
        """
        self.Model.objects.create(f1='a', f2='b')
        self.Model.objects.create(f1='a', f2='b')
        with captured_stderr():
            with self.assertRaises(IntegrityError):
                self.ut.field_defs.add(self.f1, self.f2)
    
    def test_cannot_insert_duplicate_row(self):
        """
        Inserting a duplicate rows shouldn't work
        """
        self.Model.objects.create(f1='a', f2='b')
        self.ut.field_defs.add(self.f1, self.f2)
        with captured_stderr():
            with self.assertRaises(IntegrityError):
                self.Model.objects.create(f1='a', f2='b')
    
    def test_cannot_remove_unique(self):
        """
        Removing a unique constraint that cause 
        duplicate rows shouldn't work
        """
        self.ut.field_defs.add(self.f1, self.f2)
        self.Model.objects.create(f1='a', f2='b')
        self.Model.objects.create(f1='a', f2='c')
        with captured_stderr():
            with self.assertRaises(IntegrityError):
                self.ut.field_defs.remove(self.f2)

    def test_clear_removes_unique(self):
        """
        Removing a unique constraint should relax duplicate row
        validation
        """
        self.Model.objects.create(f1='a', f2='b')
        self.ut.field_defs.add(self.f1, self.f2)
        self.ut.field_defs.clear()
        self.Model.objects.create(f1='a', f2='b')
    
class Mixin(object):
    
    def method(self):
        return 'Mixin'

class ModelProxy(CharFieldDefinition):
    
    class Meta:
        proxy = True

class ModelSubclass(models.Model):
    
    field = models.CharField(max_length=5)
    
    class Meta:
        abstract = True
    
    def method(self):
        return 'ModelSubclass'
    
class ModelSubclassWithTextField(models.Model):
    
    field = models.TextField()
    second_field = models.NullBooleanField()
    
    class Meta:
        abstract = True

class MutableModelSubclass(MutableModel):
    pass
        
class BaseDefinitionTest(BaseModelDefinitionTestCase):
    
    def test_clean(self):
        bd = BaseDefinition()
        
        # Base must be a class
        bd.base = BaseDefinitionTest.test_clean
        self.assertRaises(ValidationError, bd.clean)
        
        # Subclasses of MutableModel are not valid bases
        bd.base = MutableModelSubclass
        self.assertRaises(ValidationError, bd.clean)
        
        # Mixin objets are valid bases
        bd.base = Mixin
        bd.clean()

        # Model subclasses are valid bases if they are abstract
        bd.base = ModelSubclass
        bd.clean()
        
        # Model subclasses that are not abstract are invalid
        bd.base = CharFieldDefinition
        self.assertRaises(ValidationError, bd.clean)
        bd.base = ModelProxy
        self.assertRaises(ValidationError, bd.clean)
    
    def test_base_inheritance(self):
        Model = self.model_def.model_class()
        
        BaseDefinition.objects.create(model_def=self.model_def,
                                      base=Mixin)
        self.assertTrue(issubclass(Model, Mixin))
        
        BaseDefinition.objects.create(model_def=self.model_def,
                                      base=ModelSubclass)
        self.assertTrue(issubclass(Model, Mixin) and
                        issubclass(Model, ModelSubclass))
    
    def test_base_ordering(self):
        Model = self.model_def.model_class()
        
        BaseDefinition.objects.create(model_def=self.model_def,
                                      base=Mixin, order=2)
        model_subclass_def = BaseDefinition.objects.create(model_def=self.model_def,
                                                           base=ModelSubclass,
                                                           order=1)
        
        instance = Model()
        self.assertEqual('ModelSubclass', instance.method())
        
        model_subclass_def.order = 3
        model_subclass_def.save()
        instance = Model()
        self.assertEqual('Mixin', instance.method())
        
    def test_abstract_field_inherited(self):
        bd = BaseDefinition.objects.create(model_def=self.model_def,
                                           base=ModelSubclass)
        
        Model = self.model_def.model_class()
        
        Model.objects.create(field='value')
        
        # Test column alteration and addition by replacing the base with
        # a new one with a field with the same name and a second field.
        bd.base = ModelSubclassWithTextField
        bd.save()
        Model.objects.get(field='value')
        # The original CharField should be replaced by a TextField with no
        # max_length and a second field should be added
        Model.objects.create(field='another one bites the dust',
                             second_field=True)
        
        # Test column deletion by deleting the base
        # This should cause the model to loose all it's fields and the table
        # to loose all it's columns
        bd.delete()
        self.assertEqual(list(Model.objects.values_list()),
                         [(instance.id,) for instance in Model.objects.all()])
        self.assertColumnDoesntExists(Model._meta.db_table, 'field')




from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ...models import FieldDefinition


class BooleanFieldDefinition(FieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'boolean field')
        verbose_name_plural = _(u'boolean fields')
        defined_field_class = fields.BooleanField
        defined_field_category = _(u'boolean')

class NullBooleanFieldDefinition(FieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'null boolean field')
        verbose_name_plural = _(u'null boolean fields')
        defined_field_class = fields.NullBooleanField
        defined_field_category = _(u'boolean')




def patch_db_field_compare():
    """
    Field instances cannot be compared to other objects because of attribute
    presence assumptions in it's __cmp__ method. To allow full_clean we must
    override the item_field __cmp__ method to return NotImplemented if the
    object it's compared to isn't a Field instance. Let's monkey patch it!
    see https://code.djangoproject.com/ticket/17851
    """
    from django.db.models.fields import Field
    try:
        assert Field() != None
    except AttributeError:
        del Field.__cmp__
        def _Field__lt__(self, other):
            if isinstance(other, Field):
                return self.creation_counter < other.creation_counter
            return NotImplemented
        Field.__lt__ = _Field__lt__
        assert Field() != None

def patch_postgis_bad_geomery_escape():
    """
    When using django 1.3.X with PostgreSQL 9.1 some geometry can be badly
    escaped. Monkey patch PostGISAdapter to fix this issue.
    see https://code.djangoproject.com/ticket/16778
    """
    import django
    if django.VERSION < (1, 4):
        from django.db import connections
        postgis_engine = 'django.contrib.gis.db.backends.postgis'
        if any(connection.settings_dict['ENGINE'] == postgis_engine
               for connection in connections.all()):
            from psycopg2 import Binary
            from django.contrib.gis.db.backends.postgis.adapter import PostGISAdapter
            
            PostGISAdapter__init__ = PostGISAdapter.__init__
            def _PostGISAdapter__init__(self, *args, **kwargs):
                PostGISAdapter__init__(self, *args, **kwargs)
                self._adapter = Binary(self.ewkb)
            PostGISAdapter.__init__ = _PostGISAdapter__init__
            
            def _PostGISAdapter_prepare(self, conn):
                self._adapter.prepare(conn)
            PostGISAdapter.prepare = _PostGISAdapter_prepare
            
            def _PostGISAdapter_getquoted(self):
                return 'ST_GeomFromEWKB(%s)' % self._adapter.getquoted()
            PostGISAdapter.getquoted = _PostGISAdapter_getquoted




from ...test.testcases import FieldDefinitionTestMixin
from ...tests.models.utils import BaseModelDefinitionTestCase

from .models import (EmailFieldDefinition, IPAddressFieldDefinition,
    SlugFieldDefinition, URLFieldDefinition)


class EmailFieldDefinitionTest(FieldDefinitionTestMixin,
                               BaseModelDefinitionTestCase):
    field_definition_cls = EmailFieldDefinition
    field_values = ('guido@python.org', 'god@heaven.com')
    
class IPAddressFieldDefinitionTest(FieldDefinitionTestMixin,
                                   BaseModelDefinitionTestCase):
    field_definition_cls = IPAddressFieldDefinition
    field_defintion_init_kwargs = {'default': '192.168.1.1'}
    field_values = ('127.0.0.1', '82.94.164.162')

class SlugFieldDefinitionTest(FieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = SlugFieldDefinition
    field_values = (
        'an-awesome-slug_-_-',
        '2012-4-7-so-late'
    )
    
class URLFieldDefinitionTest(FieldDefinitionTestMixin,
                             BaseModelDefinitionTestCase):
    field_definition_cls = URLFieldDefinition
    field_values = (
        'https://github.com/charettes/django-mutant',
        'http://travis-ci.org/#!/charettes/django-mutant',
    )

try:
    from .models import GenericIPAddressFieldDefinition
except ImportError:
    pass
else:
    class GenericIPAddressFieldDefinitionTest(FieldDefinitionTestMixin,
                                              BaseModelDefinitionTestCase):
        field_definition_cls = GenericIPAddressFieldDefinition
        field_values = (
            '127.0.0.1',
            '2001:db8:85a3::8a2e:370:7334'
        )




from django.contrib.gis.db import models
from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _

from ....management import FIELD_DEFINITION_POST_SAVE_UID
from ....models import FieldDefinition, FieldDefinitionBase

from ..management import geometry_field_definition_post_save


class GeometryFieldDefinitionBase(FieldDefinitionBase):
    """
    Replace `post_save` connected signal by a wrapper that execute deferred sql
    required for some geometry field creation when relying on south
    """
    def __new__(cls, name, parents, attrs):
        definition = super(GeometryFieldDefinitionBase, cls).__new__(cls, name, parents, attrs)
        model = definition._meta.object_name.lower()
        post_save.disconnect(sender=definition,
                             dispatch_uid=FIELD_DEFINITION_POST_SAVE_UID % model)
        post_save.connect(geometry_field_definition_post_save, definition)
        return definition

DIM_CHOICES = (
    (2, _(u'Two-dimensional')),
    (3, _(u'Three-dimensional')),
)

srid_help_text = _(u'Spatial Reference System Identity')

spatial_index_help_text = _(u'Creates a spatial index for the given '
                            u'geometry field.')

dim_help_text = _(u'Coordinate dimension.')

geography_help_text = _(u'Creates a database column of type geography, '
                        u'rather than geometry.')

class GeometryFieldDefinition(FieldDefinition):
    
    __metaclass__ = GeometryFieldDefinitionBase
    
    srid = models.IntegerField(_(u'SRID'), default=4326,
                               help_text=srid_help_text)
    spatial_index = models.BooleanField(_(u'spatial index'), default=True,
                                        help_text=spatial_index_help_text)
    dim = models.PositiveSmallIntegerField(_(u'coordinate dimension'),
                                           choices=DIM_CHOICES, default=2,
                                           help_text=dim_help_text)
    geography = models.BooleanField(_(u'geography'), default=False,
                                    help_text=geography_help_text)
    
    class Meta:
        app_label = 'mutant'
        defined_field_class = models.GeometryField
        defined_field_options = ('srid', 'spatial_index', 'dim', 'geography')
        defined_field_category = _(u'geometry')

class PointFieldDefinition(GeometryFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'point field')
        verbose_name_plural = _(u'point fields')
        defined_field_class = models.PointField
        
class LineStringFieldDefinition(GeometryFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'line string field')
        verbose_name_plural = _(u'line string fields')
        defined_field_class = models.LineStringField
        
class PolygonFieldDefinition(GeometryFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'polygon field')
        verbose_name_plural = _(u'polygon fields')
        defined_field_class = models.PolygonField

class MultiPointFieldDefinition(GeometryFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'multi point field')
        verbose_name_plural = _(u'multi point fields')
        defined_field_class = models.MultiPointField
        
class MultiLineStringFieldDefinition(GeometryFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'multi line string field')
        verbose_name_plural = _(u'multi line string fields')
        defined_field_class = models.MultiLineStringField

class MultiPolygonFieldDefinition(GeometryFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'multi polygon field')
        verbose_name_plural = _(u'multi polygon fields')
        defined_field_class = models.MultiPolygonField

class GeometryCollectionFieldDefinition(GeometryFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'geometry collection field')
        verbose_name_plural = _(u'geometry collection fields')
        defined_field_class = models.GeometryCollectionField




from ....management import field_definition_post_save, perform_ddl


def geometry_field_definition_post_save(sender, instance, created, raw, **kwargs):
    """
    Geometry fields need execution of deferred sql on creation. We make sure to
    clear pending deferred sql in order to avoid executing it now.
    """
    model_class = instance.model_def.model_class()
    
    perform_ddl(model_class, 'clear_deferred_sql')
    
    field_definition_post_save(sender, instance, created, raw, **kwargs)
    
    perform_ddl(model_class, 'execute_deferred_sql')



from inspect import isclass

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError, ImproperlyConfigured
from django.db import models
from django.db.models.fields import FieldDoesNotExist
from django.db.models.loading import cache as model_cache
from django.db.models.sql.constants import LOOKUP_SEP
from django.utils.translation import ugettext_lazy as _
from orderable.models import OrderableModel
from picklefield.fields import PickledObjectField

from ..db.fields import (LazilyTranslatedField,
    PythonIdentifierField)
from ..db.models import MutableModel


def _get_db_table(app_label, model):
    return "mutant_%s_%s" % (app_label, model)

def _remove_from_model_cache(model_class):
    try:
        opts = model_class._meta
    except AttributeError:
        return
    app_label, model_name = opts.app_label, opts.object_name.lower()
    with model_cache.write_lock:
        app_models = model_cache.app_models.get(app_label, False)
        if app_models:
            model = app_models.pop(model_name, False)
            if model:
                model_cache._get_models_cache.clear()
                model._is_obsolete = True
                return model

class _ModelClassProxy(object):

    def __init__(self, model_class):
        self.__dict__['model_class'] = model_class

    @property
    def __model_class_is_obsolete(self):
        return self.model_class.is_obsolete()

    @staticmethod
    def __get_underlying_model_class(value):
        if isinstance(value, _ModelClassProxy):
            return value.model_class
        elif isclass(value) and issubclass(value, MutableModel):
            return value

    def __get_model_class(self):
        if self.__model_class_is_obsolete:
            return self.__get__(None, None)
        else:
            return self.model_class

    def __get__(self, instance, owner):
        if self.__model_class_is_obsolete:
            try:
                definition = self.model_class.definition()
            except ModelDefinition.DoesNotExist:
                raise AttributeError('This model definition has been deleted')
            else:
                self.__dict__['model_class'] = definition.model_class()
        return self.model_class

    def __set__(self, instance, value):
        model_class = self.__get_underlying_model_class(value)
        if model_class is not None:
            self.model_class = model_class
        else:
            raise AttributeError('Invalid value')

    def __call__(self, *args, **kwargs):
        model_class = self.__get_model_class()
        return model_class(*args, **kwargs)

    def __getattr__(self, name):
        model_class = self.__get_model_class()
        return getattr(model_class, name)

    def __setattr__(self, name, value):
        model_class = self.__get_model_class()
        return setattr(model_class, name, value)

    def __delattr__(self, name):
        model_class = self.__get_model_class()
        return delattr(model_class, name)

    def __instancecheck__(self, instance):
        model_class = self.__get_model_class()
        return isinstance(instance, model_class)

    def __eq__(self, other):
        other_model_class = self.__get_underlying_model_class(other)
        if type(self.model_class) == type(other_model_class):
            return self.model_class == other_model_class
        else:
            return NotImplemented
        
    def __str__(self):
        model_class = self.__get_model_class()
        return str(model_class)

class ModelDefinition(ContentType):
    
    object_name = PythonIdentifierField(_(u'object name'))
    
    verbose_name = LazilyTranslatedField(_(u'verbose name'),
                                         blank=True, null=True)
    
    verbose_name_plural = LazilyTranslatedField(_(u'verbose name plural'),
                                                blank=True, null=True)
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'model definition')
        verbose_name_plural = _(u'model definitions')
    
    def __init__(self, *args, **kwargs):
        super(ModelDefinition, self).__init__(*args, **kwargs)
        if self.pk:
            self._model_class = super(ModelDefinition, self).model_class()
    
    def get_model_bases(self):
        return tuple(bd.base for bd in self.basedefinitions.all())
    
    def get_model_opts(self):
        attrs = {
            'app_label': self.app_label,
            'db_table': _get_db_table(*self.natural_key()),
        }
        
        if self.verbose_name is not None:
            attrs['verbose_name'] = self.verbose_name
            
        if self.verbose_name_plural is not None:
            attrs['verbose_name_plural'] = self.verbose_name_plural
        
        unique_together = tuple(tuple(utd.field_defs.names())
                                    for utd in self.uniquetogetherdefinitions.all())
        if unique_together:
            attrs['unique_together'] = unique_together
        
        ordering = tuple(ordf.get_defined_ordering()
                            for ordf in self.orderingfielddefinitions.all())
        if ordering:
            # Make sure not to add ordering if it's empty since it would
            # prevent the model from inheriting it's possible base ordering.
            # Kinda related to django #17429
            attrs['ordering'] = ordering
        
        return type('Meta', (), attrs)
    
    def get_model_attrs(self, existing_model_class=None):
        attrs = {
            'Meta': self.get_model_opts(),
            '__module__': "mutant.apps.%s.models" % self.app_label,
            '_definition': (self.__class__, self.pk),
            '_is_obsolete': False,
        }
        
        attrs.update(dict((f.name, f.field_instance())
                            for f in self.fielddefinitions.select_subclasses()))
        
        return attrs
    
    def _create_model_class(self, existing_model_class=None):
        bases = self.get_model_bases()
        bases += (MutableModel,)
            
        attrs = self.get_model_attrs(existing_model_class)
        
        _remove_from_model_cache(existing_model_class)
        
        model = type(str(self.object_name), bases, attrs)
        
        return model

    def model_class(self, force_create=False):
        existing_model_class = super(ModelDefinition, self).model_class()
        if force_create:
            model_class = self._create_model_class(existing_model_class)
        else:
            model_class = existing_model_class
            if model_class is None:
                model_class = self._create_model_class()
        return _ModelClassProxy(model_class)
    
    @property
    def model_ct(self):
        content_type = getattr(self, '_contenttype_ptr_cache', None)
        if content_type is None:
            content_type = ContentType.objects.get(id=self.contenttype_ptr_id)
            self._contenttype_ptr_cache = content_type
        return self._contenttype_ptr_cache
    
    def clean(self):
        """
        Ensure app_label doesn't override an installed app one
        since model collision could occur and would cause a lot of
        side effects, i. e.:
        
        Defining a new auth.User, while not tested, could override
        the existing one and create a beautiful mess in django's
        internals
        """
        try:
            models.loading.cache.get_app(self.app_label, emptyOK=True)
        except ImproperlyConfigured:
            pass
        else:
            msg = _(u'Cannot cloak an installed app')
            raise ValidationError({'label': [msg]})
    
    def save(self, *args, **kwargs):
        self.model = self.object_name.lower()
        
        save = super(ModelDefinition, self).save(*args, **kwargs)
        
        self._model_class = super(ModelDefinition, self).model_class()
        
        return save
    
    def delete(self, *args, **kwargs):
        model_class = self.model_class()
        
        delete = super(ModelDefinition, self).delete(*args, **kwargs)
        
        ContentType.objects.clear_cache()
        
        _remove_from_model_cache(model_class)
        
        del self._model_class
        
        return delete
        
    def __unicode__(self):
        return u'.'.join((self.app_label, self.object_name))

class ModelDefinitionAttribute(models.Model):
    """
    A mixin used to make sure models that alter the state of a defined model
    clear the cached version
    """
    
    model_def = models.ForeignKey(ModelDefinition, related_name="%(class)ss")
    
    class Meta:
        abstract = True
    
    def save(self, *args, **kwargs):
        save = super(ModelDefinitionAttribute, self).save(*args, **kwargs)
        self.model_def.model_class(force_create=True)
        return save
    
    def delete(self, *args, **kwargs):
        delete = super(ModelDefinitionAttribute, self).delete(*args, **kwargs)
        self.model_def.model_class(force_create=True)
        return delete

class BaseDefinition(OrderableModel, ModelDefinitionAttribute):
    """
    Model used to represent bases of a ModelDefinition
    """
    
    base = PickledObjectField(_(u'base'))
    
    class Meta:
        app_label = 'mutant'
        ordering = ('order',)
        unique_together = (('model_def', 'order'),)
    
    def clean(self):
        # TODO: Validate the whole inheritance field clashes
        base = self.base
        msg = None
        if isclass(base):
            if issubclass(base, models.Model):
                opts = base._meta
                if not opts.abstract:
                    msg = _(u'Model base must be abstract')
                elif issubclass(base, MutableModel):
                    msg = _(u'Base cannot be a subclass of MutableModel')
        else:
            msg = _(u'Base must be a class')
        if msg:
            raise ValidationError({'base': [msg]})
        return super(BaseDefinition, self).clean()

class OrderingFieldDefinition(OrderableModel, ModelDefinitionAttribute):
    
    lookup = models.CharField(max_length=255)
    
    descending = models.BooleanField(_(u'descending'), default=False)
    
    class Meta(OrderableModel.Meta):
        app_label = 'mutant'
        # TODO: Should be unique both it bugs order swapping 
        #unique_together = (('model_def', 'order'),)
    
    def clean(self):
        """
        Make sure the lookup makes sense
        """
        if self.lookup == '?': # Randomly sort
            return
        #TODO: Support order_with_respect_to...
        else:
            lookups = self.lookup.split(LOOKUP_SEP)
            opts = self.model_def.model_class()._meta
            valid = True
            while len(lookups):
                lookup = lookups.pop(0)
                try:
                    field = opts.get_field(lookup)
                except FieldDoesNotExist:
                    valid = False
                else:
                    if isinstance(field, models.ForeignKey):
                        opts = field.rel.to._meta
                    elif len(lookups): # Cannot go any deeper
                        valid = False
                finally:
                    if not valid:
                        msg = _(u"This field doesn't exist")
                        raise ValidationError({'lookup': [msg]})
    
    def get_defined_ordering(self):
        return ("-%s" % self.lookup) if self.descending else self.lookup

class UniqueTogetherDefinition(ModelDefinitionAttribute):
    
    field_defs = models.ManyToManyField('FieldDefinition',
                                        related_name='unique_together_defs')
    
    class Meta:
        app_label = 'mutant'
    
    def __unicode__(self):
        names = ', '.join(self.field_defs.names())
        return _(u"Unique together of (%s)") % names
    
    def clean(self):
        for field_def in self.field_defs.select_related('model_def'):
            if field_def.model_def != self.model_def:
                msg = _(u'All fields must be of the same model')
                raise ValidationError({'field_defs': [msg]})



import re

from django.core.validators import RegexValidator
from django.utils.translation import ugettext_lazy as _


python_identifier_re = re.compile('^[a-z_][\w_]*$', re.IGNORECASE)
validate_python_identifier = RegexValidator(python_identifier_re, _(u'Enter a valid python identifier.'), 'invalid')

python_object_path_re = re.compile('^[a-z_]+(\.[\w_]+)*$', re.IGNORECASE)
validate_python_object_path = RegexValidator(python_object_path_re, _(u'Enter a valid python object path.'), 'invalid')



from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db.models import deletion, fields
from django.utils.translation import ugettext_lazy as _
from picklefield.fields import PickledObjectField
from south.db import db as south_api

from ...db.fields import PythonIdentifierField
from ...db.models import MutableModel
from ...management import perform_ddl
from ...managers import FilteredQuerysetManager
from ...models.field import FieldDefinition
from ...models.model import ModelDefinition


related_name_help_text = _(u'The name to use for the relation from the '
                           u'related object back to this one.')

class RelatedFieldDefinition(FieldDefinition):
    
    to = fields.related.ForeignKey(ContentType, verbose_name=_(u'to'),
                                   related_name="%(app_label)s_%(class)s_to")
    
    related_name = PythonIdentifierField(_(u'related name'),
                                         blank=True, null=True,
                                         help_text=related_name_help_text)
    
    class Meta:
        app_label = 'mutant'
        abstract = True
        defined_field_options = ('related_name',)
        defined_field_category = _(u'related')
    
    @property
    def is_recursive_relationship(self):
        """
        Whether or not `to` points to this field's model definition
        """
        try:
            model_def = self.model_def
        except ModelDefinition.DoesNotExist:
            pass
        else:
            return self.to_id == model_def.contenttype_ptr_id
    
    @property
    def to_model_class_is_mutable(self):
        return issubclass(self.to.model_class(), MutableModel)
    
    def clean(self):
        if (not self.to_model_class_is_mutable and
            self.related_name is not None):
            msg = _(u'Cannot assign a related manager to non-mutable model')
            raise ValidationError({'related_name': [msg]})
    
    def get_field_options(self):
        options = super(RelatedFieldDefinition, self).get_field_options()
        
        if self.is_recursive_relationship:
            options['to'] = fields.related.RECURSIVE_RELATIONSHIP_CONSTANT
        else:
            opts = self.to.model_class()._meta
            options['to'] = "%s.%s" % (opts.app_label, opts.object_name)
            
        if not self.to_model_class_is_mutable:
            options['related_name'] = '+'
        
        return options
    
    def _south_ready_field_instance(self):
        """
        South add_column choke when passing 'self' or 'app.Model' to `to` kwarg,
        so we have to create a special version for it.
        """
        cls = self.get_field_class()
        options = self.get_field_options()
        options['to'] = self.to.model_class()
        return cls(**options)

ON_DELETE_CHOICES = (('CASCADE', _(u'CASCADE')),
                     ('PROTECT', _(u'PROTECT')),
                     ('SET_NULL', _(u'SET_NULL')),
                     ('SET_DEFAULT', _(u'SET_DEFAULT')),
                     ('SET_VALUE', _(u'SET(VALUE)')),
                     ('DO_NOTHING', _(u'DO_NOTHING')))

to_field_help_text = _(u'The field on the related object that the '
                       u'relation is to.')

on_delete_help_text = _(u'Behavior when an object referenced by this field '
                        u'is deleted')

class ForeignKeyDefinition(RelatedFieldDefinition):
    
    to_field = PythonIdentifierField(_(u'to field'), blank=True, null=True,
                                     help_text=to_field_help_text)
    
    one_to_one = fields.BooleanField(editable=False, default=False)
    
    on_delete = fields.CharField(_(u'on delete'), blank=True, null=True,
                                 choices=ON_DELETE_CHOICES, default='CASCADE',
                                 max_length=11, help_text=on_delete_help_text)
    
    on_delete_set_value = PickledObjectField(_(u'on delete set value'), null=True)
    
    objects = FilteredQuerysetManager(one_to_one=False)
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'foreign key')
        verbose_name_plural = _(u'foreign keys')
        defined_field_class = fields.related.ForeignKey
        defined_field_options = ('to_field',)
        
    def clean(self):
        try:
            super(ForeignKeyDefinition, self).clean()
        except ValidationError as e:
            messages = e.message_dict
        else:
            messages = {}
        
        if self.on_delete == 'SET_NULL':
            if not self.null:
                msg = _(u"This field can't be null")
                messages['on_delete'] = [msg]
        elif (self.on_delete == 'SET_DEFAULT' and
              self.default == fields.NOT_PROVIDED):
            msg = _(u'This field has no default value')
            messages['on_delete'] = [msg]
            
        if messages:
            raise ValidationError(messages)
        
    def get_field_options(self):
        options = super(ForeignKeyDefinition, self).get_field_options()
        if self.on_delete == 'SET_VALUE':
            on_delete = deletion.SET(self.on_delete_set_value)
        else:
            on_delete = getattr(deletion, self.on_delete, None)
        options['on_delete'] = on_delete
        return options

class OneToOneFieldDefinition(ForeignKeyDefinition):
    
    objects = FilteredQuerysetManager(one_to_one=True)
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'one to one field')
        verbose_name_plural = _(u'one to one fields')
        defined_field_class = fields.related.OneToOneField
        
    def save(self, *args, **kwargs):
        self.one_to_one = True
        return super(OneToOneFieldDefinition, self).save(*args, **kwargs)

through_help_text = _(u'Intermediary model')

db_table_help_text = _(u'The name of the table to create for storing the '
                       u'many-to-many data')

class ManyToManyFieldDefinition(RelatedFieldDefinition):
    
    symmetrical = fields.NullBooleanField(_(u'symmetrical'))
    
    through = fields.related.ForeignKey(ContentType, blank=True, null=True,
                                        related_name="%(app_label)s_%(class)s_through",
                                        help_text=through_help_text)
    # TODO: This should not be a SlugField
    db_table = fields.SlugField(max_length=30, blank=True, null=True,
                                help_text=db_table_help_text)
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'many to many field')
        verbose_name_plural = _(u'many to many fields')
        defined_field_class = fields.related.ManyToManyField
        defined_field_options = ('symmetrical', 'through', 'db_table')

    def clean(self):
        try:
            super(ManyToManyFieldDefinition, self).clean()
        except ValidationError as e:
            messages = e.message_dict
        else:
            messages = {}
        
        if (self.symmetrical is not None and 
            not self.is_recursive_relationship):
            msg = _(u"The relationship can only be symmetrical or not if it's "
                    u"recursive, i. e. it points to 'self'")
            messages['symmetrical'] = [msg]
        
        if self.through:
            if self.db_table:
                msg = _(u'Cannot specify a db_table if an intermediate '
                        u'model is used.')
                messages['db_table'] = [msg]
        
            if self.symmetrical:
                msg = _(u'Many-to-many fields with intermediate model cannot '
                        u'be symmetrical.')
                messages.setdefault('symmetrical', []).append(msg)
            
            seen_from, seen_to = 0, 0
            to_model = self.to.model_class()  
            through_class = self.through.model_class()
            from_model = self.model_def.cached_model
            for field in through_class._meta.fields:
                rel_to = getattr(field.rel, 'to', None)
                if rel_to == from_model:
                    seen_from += 1
                elif rel_to == to_model:
                    seen_to += 1
            if self.is_recursive_relationship():
                if seen_from > 2:
                    msg = _(u'Intermediary model %s has more than two foreign '
                            u'keys to %s, which is ambiguous and is not permitted.')
                    formated_msg = msg % (through_class._meta.object_name,
                                          from_model._meta.object_name)
                    messages.setdefault('through', []).append(formated_msg)
            else:
                msg = _(u'Intermediary model %s has more than one foreign key '
                        u' to %s, which is ambiguous and is not permitted.')
                if seen_from > 1:
                    formated_msg = msg % (through_class._meta.object_name,
                                          from_model._meta.object_name)
                    messages.setdefault('through', []).append(formated_msg)
                if seen_to > 1:
                    formated_msg = msg % (through_class._meta.object_name,
                                          to_model._meta.object_name)
                    messages.setdefault('through', []).append(formated_msg)
                
        if messages:
            raise ValidationError(messages)
        
    def save(self, *args, **kwargs):
        # TODO: This should be moved to signals
        create = not self.pk
        
        save = super(ManyToManyFieldDefinition, self).save(*args, **kwargs)
        model = self.model_def.model_class()
        field = model._meta.get_field(str(self.name))
        intermediary_model = field.rel.through

        if create:
            if self.through is None:
                opts = intermediary_model._meta
                fields = tuple((field.name, field) for field in opts.fields)
                perform_ddl(model, 'create_table', opts.db_table, fields)
        else:
            #TODO: look for db_table rename
            pass
        
        return save




from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ...models.field import FieldDefinition

class CharFieldDefinition(FieldDefinition):
    
    max_length = fields.PositiveSmallIntegerField(_(u'max length'),
                                                  blank=True, null=True)
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'char field')
        verbose_name_plural = _(u'char fields')
        defined_field_class = fields.CharField
        defined_field_options = ('max_length',)
        defined_field_category = _(u'text')
        
class TextFieldDefinition(CharFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.TextField




from django.db import connection, connections, router
from django.test.testcases import _deferredSkip

from mutant.db.models import MutableModel
from mutant.models.model import ModelDefinition
from mutant.test.testcases import (ModelDefinitionDDLTestCase,
    VersionCompatMixinTestCase)


class BaseModelDefinitionTestCase(ModelDefinitionDDLTestCase,
                                  VersionCompatMixinTestCase):
    
    def setUp(self):
        self.model_def = ModelDefinition.objects.create(app_label='app',
                                                        object_name='Model')
        
    def assertTableExists(self, db, table_name):
        tables = connections[db].introspection.table_names()
        msg = "Table '%s.%s' doesn't exist, existing tables are %s"
        self.assertTrue(table_name in tables, msg % (db, table_name, tables))
        
    def assertTableDoesntExists(self, db, table_name):
        self.assertRaises(AssertionError, self.assertTableExists, db, table_name)
        
    def _table_columns_iterator(self, table_name):
        cursor = connection.cursor()
        description = connection.introspection.get_table_description(cursor, table_name)
        return (row[0] for row in description)
    
    def assertColumnExists(self, table_name, field_name):
        fields = self._table_columns_iterator(table_name)
        self.assertTrue(field_name in fields,
                        "Field '%(table)s.%(field)s' doesn't exist, '%(table)s'"
                        "'s fields are %(fields)s" % {'table': table_name,
                                                      'field': field_name,
                                                      'fields': fields})
        
    def assertColumnDoesntExists(self, table_name, field_name):
        self.assertRaises(AssertionError, self.assertColumnExists,
                          table_name, field_name)

def _get_mutant_model_db():
    # TODO: This should rely on syncdb instead
    return router.db_for_write(MutableModel)

def skipIfMutantModelDBFeature(feature, default=False):
    db = _get_mutant_model_db()
    return _deferredSkip(lambda: getattr(connections[db].features, feature, default),
                         "Database %s has feature %s" % (db, feature))

def skipUnlessMutantModelDBFeature(feature, default=True):
    db = _get_mutant_model_db()
    return _deferredSkip(lambda: not getattr(connections[db].features, feature, default),
                         "Database %s doesn't support feature %s" % (db, feature))




from django.contrib.gis.db import models


class GeoModel(models.Model):
    """
    A model to be used as a BaseDefinition on ModelDefinition instance with
    GeometryFieldDefinition instances 
    """
    
    objects = models.GeoManager()
    
    class Meta:
        abstract = True




from django.db.models import Manager
from django.db.models.query import QuerySet

from .common import choices_from_dict

class FilteredQuerysetManager(Manager):
    
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        super(FilteredQuerysetManager, self).__init__()
        
    def get_query_set(self):
        qs = super(FilteredQuerysetManager, self).get_query_set()
        return qs.filter(*self.args, **self.kwargs)

class InheritedModelManager(Manager):
    use_for_related_fields = True
    
    class InheritanceQuerySet(QuerySet):
        """
        Based on http://goo.gl/8CM5X by Jeff Elmores
        """
        
        def select_subclasses(self, *subclasses):
            if not subclasses:
                subclasses = self.model.subclasses()
            qs = self.select_related(*subclasses)
            qs.subclasses = subclasses
            return qs
        
        def _clone(self, klass=None, setup=False, **kwargs):
            try:
                kwargs.update({'subclasses': self.subclasses})
            except AttributeError:
                pass
            cls = InheritedModelManager.InheritanceQuerySet
            return super(cls, self)._clone(klass, setup, **kwargs)
        
        def iterator(self):
            iterator = super(InheritedModelManager.InheritanceQuerySet, self).iterator()
            if getattr(self, 'subclasses', False):
                for obj in iterator:
                    yield obj.type_cast()
            else:
                # yield from iterator
                for obj in iterator:
                    yield obj
    
    def select_subclasses(self, *subclasses):
        return self.get_query_set().select_subclasses(*subclasses)
    
    def get_query_set(self):
        return self.InheritanceQuerySet(self.model, using=self._db)

class FieldDefinitionChoiceManager(Manager):
    use_for_related_fields = True
    
    class ChoiceDefinitionQuerySet(QuerySet):
        
        def as_choices(self):
            qs = self.values('group', 'value', 'label')
            return choices_from_dict(qs)
    
    def as_choices(self):
        qs = self.ChoiceDefinitionQuerySet(self.model, using=self._db)
        return qs.as_choices()




from django.contrib import admin
from django.forms.widgets import TextInput

from mutant.admin.fields import FieldDefinitionTypeField
from mutant.models.field import FieldDefinition
from mutant.models.model import ModelDefinition, UniqueTogetherDefinition

class UniqueTogetherDefinitionInline(admin.TabularInline):
    
    model = UniqueTogetherDefinition

class FieldDefinitionInline(admin.StackedInline):
    
    model = FieldDefinition
    
    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name in ('verbose_name', 'help_text'):
            kwargs['widget'] = TextInput
        return super(FieldDefinitionInline, self).formfield_for_dbfield(db_field, **kwargs)
    
    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        if db_field.name == 'field_type':
            kwargs['form_class'] = FieldDefinitionTypeField
        sup = super(FieldDefinitionInline, self)
        return sup.formfield_for_foreignkey(db_field, request, **kwargs)

class ModelDefinitionAdmin(admin.ModelAdmin):
    
    list_display = ('app_label', 'object_name')
    
    inlines = (FieldDefinitionInline, UniqueTogetherDefinitionInline,)
    
    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name in ('verbose_name', 'verbose_name_plural'):
            kwargs['widget'] = TextInput
        return super(ModelDefinitionAdmin, self).formfield_for_dbfield(db_field, **kwargs)
    
#admin.site.register(ModelDefinition, ModelDefinitionAdmin)




from test_project.settings import *


DATABASES = {
    'default': {
        'ENGINE': 'django.contrib.gis.db.backends.postgis',
        'NAME': 'mutant',
        'USER': 'postgres',
    }
}

INSTALLED_APPS.extend(['django.contrib.gis', 'mutant.contrib.geo'])




from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ...models.field import FieldDefinition


class _NumericFieldDefinition(FieldDefinition):
    
    class Meta:
        proxy = True
        defined_field_category = _(u'numeric')

class SmallIntegerFieldDefinition(_NumericFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'small integer field')
        verbose_name_plural = _(u'small integer fields')
        defined_field_class = fields.SmallIntegerField
        
class PositiveSmallIntegerFieldDefinition(_NumericFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'positive small integer field')
        verbose_name_plural = _(u'positive small integer fields')
        defined_field_class = fields.PositiveSmallIntegerField
        
class IntegerFieldDefinition(_NumericFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'integer field')
        verbose_name_plural = _(u'integer fields')
        defined_field_class = fields.IntegerField
        
class PositiveIntegerFieldDefinition(_NumericFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'positive integer field')
        verbose_name_plural = _(u'positive integer fields')
        defined_field_class = fields.PositiveIntegerField

class BigIntegerFieldDefinition(_NumericFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'big integer field')
        verbose_name_plural = _(u'big integer fields')
        defined_field_class = fields.BigIntegerField
        
class FloatFieldDefinition(_NumericFieldDefinition):
    
    class Meta:
        app_label = 'mutant'
        proxy = True
        verbose_name = _(u'float field')
        verbose_name_plural = _(u'float fields')
        defined_field_class = fields.FloatField

max_digits_help_text = _(u'The maximum number of digits allowed in the number. '
                         u'Note that this number must be greater than or equal '
                         u'to ``decimal_places``, if it exists.')

decimal_places_help_text = _(u'The number of decimal places to store '
                             u'with the number.')

class DecimalFieldDefinition(_NumericFieldDefinition):
    
    max_digits = fields.PositiveSmallIntegerField(_(u'max digits'),
                                                  help_text=max_digits_help_text)
    decimal_places = fields.PositiveSmallIntegerField(_(u'decimal_places'),
                                                      help_text=decimal_places_help_text)

    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'decimal field')
        verbose_name_plural = _(u'decimal fields')
        defined_field_class = fields.DecimalField
        defined_field_options = ('max_digits', 'decimal_places',)




from django.contrib.contenttypes.models import ContentType
from django.forms.models import ModelChoiceField
from django.utils.text import capfirst

from mutant.common import choices_from_dict
from mutant.models.field import FieldDefinitionBase

class FieldDefinitionTypeField(ModelChoiceField):
    
    def __init__(self, *args, **kwargs):
        super(FieldDefinitionTypeField, self).__init__(*args, **kwargs)
        fcs = FieldDefinitionBase._field_classes
        choices = ({'value': (ContentType.objects.filter(app_label=fdc._meta.app_label,
                                                        model=fdc._meta.object_name.lower())
                                        .values_list('id', flat=True)[0]),
                    'label': capfirst(fdc.get_field_description()),
                    'group': capfirst(fdc._meta.defined_field_category)}
                        for fc, fdc in fcs.iteritems())
        self._choices = ((u"", self.empty_label),) + tuple(choices_from_dict(choices))



from django.db.models.fields import TextField
from django.utils.encoding import smart_unicode
from django.utils.functional import Promise
from django.utils.translation import ugettext, ugettext_lazy as _


class LazilyTranslatedField(TextField):
    
    def to_python(self, value):
        if (isinstance(value, Promise) and
            value._proxy____func == ugettext) or value is None:
            return value
        return _(smart_unicode(value))
    
    def get_prep_value(self, value):
        if value is None:
            return value
        elif isinstance(value, basestring):
            return smart_unicode(value)
        # We assume it's a ugettext promise
        return smart_unicode(value._proxy____args[0])




from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core import exceptions
from django.db.models import fields
from django.utils.translation import ugettext_lazy as _


class FieldDefinitionTypeField(fields.related.ForeignKey):
    
    def __init__(self, *args, **kwargs):
        defaults={'to': ContentType}
        defaults.update(kwargs)
        super(FieldDefinitionTypeField, self).__init__(*args, **defaults)
        
    def validate(self, value, model_instance):
        super(FieldDefinitionTypeField, self).validate(value, model_instance)
        if value is None:
            return
        if isinstance(value, int):
            try:
                value = ContentType.objects.get(id=value)
            except ContentType.DoesNotExist:
                msg = self.error_messages['invalid'] % {
                'model': self.rel.to._meta.verbose_name, 'pk': value}
                raise exceptions.ValidationError(msg)
        # Lazily import to avoid circular reference
        from mutant.models import FieldDefinition
        klass = value.model_class()
        if not issubclass(klass, FieldDefinition) or klass == FieldDefinition:
            msg = _(u'This field must be the ContentType of '
                    u'an explicit FieldDefinition subclass.')
            raise exceptions.ValidationError(msg)
        
class ProxyAwareGenericForeignKey(GenericForeignKey):
    """
    Basically a GenericForeignKey that saves the actal ContentType of the object
    even if it's a proxy Model.
    """
    
    def get_content_type(self, obj=None, **kwargs):
        if obj and obj._meta.proxy:
            opts = obj._meta
            natural_key = (opts.app_label, opts.object_name.lower())
            return ContentType.objects.db_manager(obj._state.db).get_by_natural_key(*natural_key)
        else:
            return super(ProxyAwareGenericForeignKey, self).get_content_type(obj, **kwargs)




from ...test.testcases import FieldDefinitionTestMixin
from ...tests.models.utils import BaseModelDefinitionTestCase

from .models import BooleanFieldDefinition, NullBooleanFieldDefinition


class CharFieldDefinitionTest(FieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = BooleanFieldDefinition
    field_defintion_init_kwargs = {'default': True}
    field_values = (True, False)

class NullBooleanFieldDefinitionTest(FieldDefinitionTestMixin,
                                     BaseModelDefinitionTestCase):
    field_definition_cls = NullBooleanFieldDefinition
    field_values = (True, None)




from django.contrib.contenttypes.models import ContentType
from django.db import connections, models, router
from django.db.models.fields import FieldDoesNotExist
from django.db.models.signals import m2m_changed, post_delete, post_save, pre_delete
from south.db import dbs

from mutant.models import (ModelDefinition, BaseDefinition, FieldDefinition,
    UniqueTogetherDefinition)


def allow_syncdbs(model):
    for db in connections:
        if router.allow_syncdb(db, model):
            yield dbs[db]
        
def perform_ddl(model, action, *args, **kwargs):
    for db in allow_syncdbs(model):
        getattr(db, action)(*args, **kwargs)

def model_definition_post_save(sender, instance, created, raw, **kwargs):
    if raw:
        ct_db = instance._state.db # cts should be one the same db as mds
        ct = ContentType.objects.using(ct_db).get(pk=instance.pk)
        instance.app_label, instance.model = ct.app_label, ct.model
    model_class = instance.model_class(force_create=True)
    opts = model_class._meta
    if created:
        fields = tuple((field.name, field) for field in opts.fields)
        perform_ddl(model_class, 'create_table', opts.db_table, fields)
    else:
        old_opts = instance._model_class._meta
        if old_opts.db_table != opts.db_table:
            perform_ddl(model_class, 'rename_table', old_opts.db_table, opts.db_table)
            # It means that the natural key has changed
            ContentType.objects.clear_cache()

post_save.connect(model_definition_post_save, ModelDefinition,
                  dispatch_uid='mutant.management.model_definition_post_save')

def model_definition_post_delete(sender, instance, **kwargs):
    model_class = instance.model_class()
    table_name = model_class._meta.db_table
    perform_ddl(model_class, 'delete_table', table_name)
    
post_delete.connect(model_definition_post_delete, ModelDefinition,
                    dispatch_uid='mutant.management.model_definition_post_delete')

def base_definition_post_save(sender, instance, created, raw, **kwargs):
    base = instance.base
    if issubclass(base, models.Model):
        model_class = instance.model_def.model_class()
        opts = model_class._meta
        table_name = opts.db_table
        if created:
            for field in base._meta.fields:
                perform_ddl(model_class, 'add_column', table_name,
                            field.name, field, keep_default=False)
        else:
            for field in base._meta.fields:
                try:
                    old_field = opts.get_field(field.name)
                except FieldDoesNotExist:
                    perform_ddl(model_class, 'add_column', table_name,
                                field.name, field, keep_default=False)
                else:
                    column = old_field.get_attname_column()[1]
                    perform_ddl(model_class, 'alter_column', table_name,
                                column, field)
                    
post_save.connect(base_definition_post_save, BaseDefinition,
                  dispatch_uid='mutant.management.base_definition_post_save')

def base_definition_pre_delete(sender, instance, **kwargs):
    """
    This is used to pass data required for deletion to the post_delete
    signal that is no more available thereafter.
    """
    if issubclass(instance.base, models.Model):
        model_class = instance.model_def.model_class()
        instance._state._deletion = (
            allow_syncdbs(model_class),
            model_class._meta.db_table,
        )

pre_delete.connect(base_definition_pre_delete, BaseDefinition,
                   dispatch_uid='mutant.management.base_definition_pre_delete')

def base_definition_post_delete(sender, instance, **kwargs):
    if issubclass(instance.base, models.Model):
        syncdbs, table_name = instance._state._deletion
        for field in instance.base._meta.fields:
            for db in syncdbs:
                db.delete_column(table_name, field.name)
        del instance._state._deletion

post_delete.connect(base_definition_post_delete, BaseDefinition,
                    dispatch_uid='mutant.management.base_definition_post_delete')

def unique_together_field_defs_changed(instance, action, model, **kwargs):
    columns = list(instance.field_defs.names())
    # If there's no columns and action is post_clear there's nothing to do
    if columns and action != 'post_clear':
        model_class = instance.model_def.model_class()
        table_name = model_class._meta.db_table
        if action in ('pre_add', 'pre_remove', 'pre_clear'):
            perform_ddl(model_class, 'delete_unique', table_name, columns)
        # Safe guard against m2m_changed.action API change
        elif action in ('post_add', 'post_remove'):
            perform_ddl(model_class, 'create_unique', table_name, columns)
            
m2m_changed.connect(unique_together_field_defs_changed,
                    UniqueTogetherDefinition.field_defs.through,
                    dispatch_uid='mutant.management.unique_together_field_defs_changed')

def field_definition_post_save(sender, instance, created, raw, **kwargs):
    """
    This signal is connected by all FieldDefinition subclasses
    see comment in FieldDefinitionBase for more details
    """
    model_class = instance.model_def.model_class()
    table_name = model_class._meta.db_table
    field = instance._south_ready_field_instance()
    
    if created:
        if hasattr(instance._state, '_creation_default_value'):
            field.default = instance._state._creation_default_value
            delattr(instance._state, '_creation_default_value')
            keep_default = False
        else:
            keep_default = True
        perform_ddl(model_class, 'add_column', table_name,
                    instance.name, field, keep_default=keep_default)
    else:
        __, column = field.get_attname_column()
        old_field = instance._old_field
        
        # Field renaming
        old_column = old_field.get_attname_column()[1]
        if column != old_column:
            perform_ddl(model_class, 'rename_column', table_name, old_column, column)
        
        # Create/Drop unique and primary key
        for opt in ('primary_key', 'unique'):
            value = getattr(field, opt)
            if value != getattr(old_field, opt):
                action_prefix = 'create' if value else 'delete'
                action = "%s_%s" % (action_prefix, opt)
                perform_ddl(model_class, action, table_name, (column,))

        perform_ddl(model_class, 'alter_column', table_name, column, field)

FIELD_DEFINITION_POST_SAVE_UID = "mutant.management.%s_post_save"

def field_definition_pre_delete(sender, instance, **kwargs):
    model_class = instance.model_def.model_class()
    opts = model_class._meta
    instance._state._deletion = (
        allow_syncdbs(model_class),
        opts.db_table,
        opts.get_field(instance.name).column
    )

pre_delete.connect(field_definition_pre_delete, sender=FieldDefinition,
                   dispatch_uid='mutant.management.field_definition_pre_delete')

def field_definition_post_delete(sender, instance, **kwargs):
    syncdbs, table_name, name = instance._state._deletion
    for db in syncdbs:
        db.delete_column(table_name, name)
    del instance._state._deletion

post_delete.connect(field_definition_post_delete, sender=FieldDefinition,
                    dispatch_uid='mutant.management.field_definition_post_delete')



#!/usr/bin/python
import re
from setuptools import setup, find_packages

from mutant import __version__

github_url = 'https://github.com/charettes/django-mutant'
long_desc = open('README.md').read()

LINK_REQUIREMENT = re.compile(r'^(?P<link>https://.+#egg=(?P<package>.+)-(?P<version>\d(?:\.\d)*))$')

install_requires = []
dependency_links = []

for requirement in (l.strip() for l in open('requirements/stable.txt')):
    match = LINK_REQUIREMENT.match(requirement)
    if match:
        install_requires.append("%(package)s==%(version)s" % match.groupdict())
        dependency_links.append(match.group('link'))
    else:
        install_requires.append(requirement)

setup(
    name='django-mutant',
    version='.'.join(str(v) for v in __version__),
    description='Dynamic model definition and alteration (evolving schemas)',
    long_description=open('README.md').read(),
    url=github_url,
    author='Simon Charette',
    author_email='charette.s@gmail.com',
    install_requires=install_requires,
    dependency_links=dependency_links,
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    license='MIT License',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



from decimal import Decimal

from ...test.testcases import FieldDefinitionTestMixin
from ...tests.models.utils import BaseModelDefinitionTestCase

from .models import (BigIntegerFieldDefinition, DecimalFieldDefinition,
    FloatFieldDefinition, IntegerFieldDefinition,
    PositiveIntegerFieldDefinition, PositiveSmallIntegerFieldDefinition,
    SmallIntegerFieldDefinition)


class SmallIntegerFieldDefinitionTest(FieldDefinitionTestMixin,
                                      BaseModelDefinitionTestCase):
    field_definition_cls = SmallIntegerFieldDefinition
    field_defintion_init_kwargs = {'default': 0}
    field_values = (-134, 245)

class PositiveSmallIntegerFieldDefinitionTest(FieldDefinitionTestMixin,
                                              BaseModelDefinitionTestCase):
    field_definition_cls = PositiveSmallIntegerFieldDefinition
    field_defintion_init_kwargs = {'default': 0}
    field_values = (135, 346)

class IntegerFieldDefinitionTest(FieldDefinitionTestMixin,
                             BaseModelDefinitionTestCase):
    field_definition_cls = IntegerFieldDefinition
    field_defintion_init_kwargs = {'default': 0}
    field_values = (44323423, -4223423)

class PositiveIntegerFieldDefinitionTest(FieldDefinitionTestMixin,
                                         BaseModelDefinitionTestCase):
    field_definition_cls = PositiveIntegerFieldDefinition
    field_defintion_init_kwargs = {'default': 0}
    field_values = (44323423, 443234234)
    
class BigIntegerFieldDefinitionTest(FieldDefinitionTestMixin,
                                    BaseModelDefinitionTestCase):
    field_definition_cls = BigIntegerFieldDefinition
    field_defintion_init_kwargs = {'default': 0}
    field_values = (443234234324, 443234234998)

class FloatFieldDefinitionTest(FieldDefinitionTestMixin,
                               BaseModelDefinitionTestCase):
    field_definition_cls = FloatFieldDefinition
    field_defintion_init_kwargs = {'default': 0}
    field_values = (1234567.84950, 18360935.1854195)

class DecimalFieldDefinitionTest(FieldDefinitionTestMixin,
                                 BaseModelDefinitionTestCase):
    field_definition_cls = DecimalFieldDefinition
    field_defintion_init_kwargs = {
        'default': 0,
        'max_digits': 15,
        'decimal_places': 7
    }
    field_values = (
        Decimal('1234567.84950'),
        Decimal('18360935.1854195'),
    )



import datetime

from ...test.testcases import FieldDefinitionTestMixin
from ...tests.models.utils import BaseModelDefinitionTestCase

from .models import (DateFieldDefinition, DateTimeFieldDefinition,
    TimeFieldDefinition)


class DateFieldDefinitionTest(FieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = DateFieldDefinition
    field_defintion_init_kwargs = {'default': datetime.date(1990, 8, 31)}
    field_values = (
        datetime.date.today(),
        datetime.date(1988, 5, 15)
    )

class DateTimeFieldDefinitionTest(FieldDefinitionTestMixin,
                                  BaseModelDefinitionTestCase):
    field_definition_cls = DateTimeFieldDefinition
    field_defintion_init_kwargs = {'default': datetime.datetime(1990, 8, 31, 23, 46)}
    field_values = (
        datetime.datetime(2020, 11, 15, 15, 34),
        datetime.datetime(1988, 5, 15, 15, 30)
    )
    
class TimeFieldDefinitionTest(FieldDefinitionTestMixin,
                                     BaseModelDefinitionTestCase):
    field_definition_cls = TimeFieldDefinition
    field_defintion_init_kwargs = {'default': datetime.time(1, 1)}
    field_values = (
        datetime.time(1, 58, 37),
        datetime.time(17, 56)
    )




from django.db.models import fields

from dynamodef.models import FieldDefinition

class FieldDefinitionProxy(FieldDefinition):
    
    class Meta:
        proxy = True
#        defined_field_class = fields.CharField
        
class FieldDefinitionSubclass(FieldDefinition):
    
    option = fields.BooleanField()
    fk = fields.related.ForeignKey(FieldDefinitionProxy, related_name='fks')
    m2m = fields.related.ManyToManyField(FieldDefinitionProxy, related_name='m2ms')

    class Meta:
#        defined_field_class = fields.CharField
        defined_field_options = ('option', 'fk', 'm2m')
        
class AbstractFieldDefinition(FieldDefinition):
    
    useless_option = fields.BooleanField()
    
    class Meta:
        abstract = True
        defined_field_options = ('useless_option',)
        
class AbstractFieldDefinitionSubclass(FieldDefinition):
    
    non_option_field = fields.BooleanField()
    other_option = fields.BooleanField()
    
    class Meta:
        abstract = True
        defined_field_options = ('other_option',)

#class MutipleInheritanceFieldDefinition(AbstractFieldDefinitionSubclass,
#                                        FieldDefinitionSubclass):
#    pass



# -*- coding: utf-8 -*-

from django.db import connection
from django.db.utils import DatabaseError
from django.utils.unittest.case import skipUnless

from ...test.testcases import FieldDefinitionTestMixin
from ...tests.models.utils import BaseModelDefinitionTestCase

from .models import CharFieldDefinition, TextFieldDefinition


class CharFieldDefinitionTest(FieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = CharFieldDefinition
    field_defintion_init_kwargs = {'max_length': 255}
    field_values = ('Raptor Jesus', 'Nirvana')
    
    @skipUnless(connection.settings_dict['ENGINE'] != 'django.db.backends.sqlite3',
                "Skipping because sqlite3 doesn't enforce CHAR length")
    def test_field_max_length(self):
        self.field.max_length = 24
        self.field.save()
        Model = self.model_def.model_class()
        with self.assertRaises(DatabaseError):
            Model.objects.create(field='Simon' * 5)

class TextFieldDefinitionTest(FieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = TextFieldDefinition
    field_values = (
        u"""
        J'ai caché
        Mieux que partout ailleurs
        Au grand jardin de mon coeur
        Une petite fleur
        Cette fleur
        Plus jolie qu'un bouquet
        Elle garde en secret
        Tous mes rêves d'enfant
        L'amour de mes parents
        Et tous ces clairs matins
        Fait d'heureux souvenirs lointains
        """,
        
        u"""
        Quand la vie
        Par moments me trahi
        Tu restes mon bonheur
        Petite fleur
        
        Sur mes vingt ans
        Je m'arrête un moment
        Pour respirer
        Le parfum que j'ai tant aimé
        
        Dans mon coeur
        Tu fleuriras toujours
        Au grand jardin d'amour
        Petite fleur
        """,
    )



import warnings

from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.contrib.contenttypes.generic import GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import signals
from django.db.models.sql.constants import LOOKUP_SEP
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _
import orderable
from picklefield.fields import dbsafe_encode, PickledObjectField

from ..db.fields import (FieldDefinitionTypeField, LazilyTranslatedField,
    ProxyAwareGenericForeignKey, PythonIdentifierField)
from ..managers import FieldDefinitionChoiceManager, InheritedModelManager

from .model import ModelDefinitionAttribute


NOT_PROVIDED = dbsafe_encode(models.NOT_PROVIDED)

def _get_concrete_model(model):
    """
    Prior to django r17573 (django 1.4), `proxy_for_model` returned the
    actual concrete model of a proxy and there was no `concrete_model`
    property so we try to fetch the `concrete_model` from the opts
    and fallback to `proxy_for_model` if it's not defined.
    """
    return getattr(model._meta, 'concrete_model', model._meta.proxy_for_model)

def _copy_fields(src, to_cls):
    """
    Returns a new instance of `to_cls` with fields data fetched from `src`.
    Useful for getting a model proxy instance from concrete model instance or
    the other way around.
    """
    fields = src._meta.fields
    data = tuple(getattr(src, field.attname) for field in fields)
    return to_cls(*data)

class FieldDefinitionBase(models.base.ModelBase):
    
    _base_definition = None
    _subclasses_lookups = []
    _proxies = {}
    _lookups = {}
    
    def __new__(cls, name, parents, attrs):
        if 'Meta' in attrs:
            Meta = attrs['Meta']
            field_class = getattr(Meta, 'defined_field_class', None)
            if field_class:
                if not issubclass(field_class, models.Field):
                    msg = ("Meta's defined_field_class must be a subclass of "
                           "django.db.models.fields.Field")
                    raise ImproperlyConfigured(msg)
                del Meta.defined_field_class
            field_options = getattr(Meta, 'defined_field_options', ())
            if field_options:
                if not isinstance(field_options, tuple):
                    msg = "Meta's defined_field_options must be a tuple"
                    raise ImproperlyConfigured(msg)
                del Meta.defined_field_options
            field_category = getattr(Meta, 'defined_field_category', None)
            if field_category:
                del Meta.defined_field_category
        else:
            field_class = None
            field_options = ()
            field_category = None
        
        definition = super(FieldDefinitionBase, cls).__new__(cls, name, parents, attrs)
        
        # Store the FieldDefinition cls
        if cls._base_definition is None:
            cls._base_definition = definition
        else:
            opts = definition._meta
            model = opts.object_name.lower()
            lookup = []
            base_definition = cls._base_definition
            parents = [definition]
            while parents:
                parent = parents.pop(0)
                if issubclass(parent, base_definition):
                    parent_opts = parent._meta
                    field_class = getattr(parent_opts, 'field_class', field_class)
                    field_category = getattr(parent_opts, 'field_category', field_category)
                    field_options += getattr(parent_opts, 'defined_field_options', ())
                    if parent is not base_definition:
                        if not (parent_opts.abstract or parent_opts.proxy):
                            lookup.insert(0, parent_opts.object_name.lower())
                        parents = list(parent._meta.parents) + parents # mimic mro
            cls._lookups[model] = lookup
            if opts.proxy:
                cls._proxies[model] = definition
            elif not opts.abstract:
                if len(lookup) == 1:
                    # TODO: #16572
                    # We can't do `select_related` on multiple one-to-one
                    # relationships...
                    # see https://code.djangoproject.com/ticket/16572
                    cls._subclasses_lookups.append(LOOKUP_SEP.join(lookup))
            
            from ..management import (field_definition_post_save,
                FIELD_DEFINITION_POST_SAVE_UID)
            object_name = definition._meta.object_name.lower()
            post_save_dispatch_uid = FIELD_DEFINITION_POST_SAVE_UID % object_name
            signals.post_save.connect(field_definition_post_save, definition,
                                      dispatch_uid=post_save_dispatch_uid)
            
            # Warn the user that they should rely on signals instead of
            # overriding the delete methods since it might not be called
            # when deleting the associated model definition.
            if definition.delete != cls._base_definition.delete:
                concrete_model = _get_concrete_model(definition)
                if (opts.proxy and
                    concrete_model.delete != cls._base_definition.delete):
                    # Because of the workaround for django #18083 in
                    # FieldDefinition, overriding the `delete` method on a proxy
                    # of a concrete FieldDefinition that also override the
                    # delete method might call some deletion code twice.
                    # Until #18083 is fixed and the workaround is removed we
                    # raise a `TypeError` to prevent this from happening.
                    msg = ("Proxy model deletion is partially broken until "
                           "django #18083 is fixed. To work around this issue, "
                           "mutant make sure to call the concrete `FieldDefinition`"
                           "you are proxying, in this case `%(concrete_cls)s`. "
                           "However, this can trigger a double execution of "
                           "`%(concrete_cls)s.delete`, thus it is prohibited.")
                    raise TypeError(msg % {'concrete_cls': concrete_model.__name__})
                def_name = definition.__name__
                warnings.warn("Avoid overriding the `delete` method on "
                              "`FieldDefinition` subclass `%s` since it won't "
                              "be called when the associated `ModelDefinition` "
                              "is deleted. If you want to perform actions on "
                              "deletion, add hooks to the `pre_delete` and "
                              "`post_delete` signals." % def_name, UserWarning)
        
        definition._meta.defined_field_class = field_class
        definition._meta.defined_field_options = tuple(set(field_options))
        definition._meta.defined_field_category = field_category
        
        return definition

class FieldDefinitionManager(InheritedModelManager):
    
    class FieldDefinitionQuerySet(InheritedModelManager.InheritanceQuerySet):
        
        def create_with_default(self, default, **kwargs):
            obj = self.model(**kwargs)
            obj._state._creation_default_value = default
            self._for_write = True
            obj.save(force_insert=True, using=self.db)
            return obj
    
    def get_query_set(self):
        return self.FieldDefinitionQuerySet(self.model, using=self._db)
    
    def names(self):
        qs = self.get_query_set()
        return qs.order_by('name').values_list('name', flat=True)
    
    def create_with_default(self, default, **kwargs):
        qs = self.get_query_set()
        return qs.create_with_default(default, **kwargs)
    
class FieldDefinition(ModelDefinitionAttribute):
    
    __metaclass__ = FieldDefinitionBase
    
    # TODO: rename field_def_type
    field_type = FieldDefinitionTypeField()
    
    name = PythonIdentifierField(_(u'name'))
    verbose_name = LazilyTranslatedField(_(u'verbose name'), blank=True, null=True)
    help_text = LazilyTranslatedField(_(u'help text'), blank=True, null=True)
    
    null = models.BooleanField(_(u'null'), default=False)
    blank = models.BooleanField(_(u'blank'), default=False)
    choices = GenericRelation('FieldDefinitionChoice',
                              content_type_field='field_def_type',
                              object_id_field='field_def_id')
    
    db_column = models.SlugField(_(u'db column'), max_length=30, blank=True, null=True)
    db_index = models.BooleanField(_(u'db index'), default=False)
    
    editable = models.BooleanField(_(u'editable'), default=True)
    default = PickledObjectField(_(u'default'), null=True, default=NOT_PROVIDED)
    
    primary_key = models.BooleanField(_(u'primary key'), default=False)
    unique = models.BooleanField(_(u'unique'), default=False)
    
    unique_for_date = PythonIdentifierField(_(u'unique for date'), blank=True, null=True)
    unique_for_month = PythonIdentifierField(_(u'unique for month'), blank=True, null=True)
    unique_for_year = PythonIdentifierField(_(u'unique for year'), blank=True, null=True)
    
    objects = FieldDefinitionManager()
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'field')
        verbose_name_plural = _(u'fields')
        unique_together = (('model_def', 'name'),)
        defined_field_options = ('name', 'verbose_name', 'help_text',
                                 'null', 'blank', 'db_column', 'db_index',
                                 'editable', 'default', 'primary_key', 'unique',
                                 'unique_for_date', 'unique_for_month', 'unique_for_year')
    
    def __init__(self, *args, **kwargs):
        super(FieldDefinition, self).__init__(*args, **kwargs)
        if self.pk and self.__class__ != FieldDefinition:
            self._old_field = self.field_instance()
    
    def save(self, *args, **kwargs):
        if not self.pk:
            app_label = self._meta.app_label
            model = self._meta.object_name.lower()
            self.field_type = ContentType.objects.get_by_natural_key(app_label, model)
            
        saved = super(FieldDefinition, self).save(*args, **kwargs)

        self._old_field = self._south_ready_field_instance()
        
        return saved
    
    def delete(self, *args, **kwargs):
        if self._meta.proxy:
            # TODO: #18083
            # Ok so this is a big issue: proxy model deletion is completely
            # broken. When you delete a inherited model proxy only the proxied
            # model is deleted, plus deletion signals are not sent for the
            # proxied model and it's subclasses. Here we attempt to fix this by
            # getting the concrete model instance of the proxy and deleting it
            # while sending proxy model signals.
            concrete_model = _get_concrete_model(self)
            concrete_model_instance = _copy_fields(self, concrete_model)
            
            # Send proxy pre_delete
            signals.pre_delete.send(self.__class__, instance=self)
            
            # Delete the concrete model
            delete = concrete_model_instance.delete(*args, **kwargs)
            
            # This should be sent before the subclasses post_delete but we
            # cannot venture into deletion.Collector to much. Better wait until
            # #18083 is fixed.
            signals.post_delete.send(self.__class__, instance=self)
            
            return delete
        return super(FieldDefinition, self).delete(*args, **kwargs)
    
    @classmethod
    def subclasses(cls):
        # TODO: rename subclasses lookups?
        return FieldDefinitionBase._subclasses_lookups
    
    def type_cast(self):
        field_type_model = self.field_type.model
        
        # Cast to the right concrete model by going up in the 
        # SingleRelatedObjectDescriptor chain
        type_casted = self
        for subclass in FieldDefinitionBase._lookups[field_type_model]:
            type_casted = getattr(type_casted, subclass)
        
        # If it's a proxy model we make to type cast it
        proxy = FieldDefinitionBase._proxies.get(field_type_model, None)
        if proxy:
            concrete_model = _get_concrete_model(proxy)
            if not isinstance(type_casted, concrete_model):
                msg = ("Concrete type casted model %s is not an instance of %s "
                       "which is the model proxied by %s" % (type_casted, concrete_model, proxy))
                raise AssertionError(msg)
            type_casted = _copy_fields(self, proxy)
        
        if type_casted._meta.object_name.lower() != field_type_model:
            raise AssertionError("Failed to type cast %s to %s" % (self, field_type_model))
        
        return type_casted
    
    @classmethod
    def get_field_class(cls):
        field_class = cls._meta.defined_field_class
        if not field_class:
            raise NotImplementedError
        return field_class
    
    @classmethod
    def get_field_description(cls):
        return capfirst(cls._meta.verbose_name)
    
    def get_field_choices(self):
        return tuple(self.choices.as_choices())
    
    def get_field_options(self):
        opts = self._meta
        options = {}
        for name in opts.defined_field_options:
            value = getattr(self, name)
            if value != opts.get_field(name).get_default():
                options[name] = value
        choices = self.get_field_choices()
        if choices:
            options['choices'] = choices
        return options
    
    def field_instance(self):
        cls = self.get_field_class()
        options = self.get_field_options()
        return cls(**options)
    
    def _south_ready_field_instance(self):
        """
        South api sometimes needs to have modified version of fields to work.
        i. e. You can't pass a ForeignKey(to='self') to add_column
        """
        return self.field_instance()
    
    def clean(self):
        # Make sure we can build the field
        try:
            field = self.field_instance()
        except Exception as e:
            raise ValidationError(e)
        else:
            # Test the specified default value
            if field.has_default():
                default = field.get_default()
                try:
                    field.clean(default, None)
                except Exception:
                    msg = _(u"%r is not a valid default value") % default
                    raise ValidationError({'default': [msg]})

class FieldDefinitionChoice(orderable.models.OrderableModel):
    """
    A Model to allow specifying choices for a field definition instance
    """
    field_def_type = FieldDefinitionTypeField(verbose_name=_(u'field_def type'))
    field_def_id = models.IntegerField(_(u'field_def def id'), db_index=True)
    field_def = ProxyAwareGenericForeignKey(ct_field='field_def_type',
                                            fk_field='field_def_id')
    
    group = LazilyTranslatedField(_(u'group'), blank=True, null=True)
    value = models.CharField(_(u'value'), max_length=255)
    label = LazilyTranslatedField(_(u'label'))
    
    objects = FieldDefinitionChoiceManager()
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'field_def choice')
        verbose_name_plural = _(u'field_def choices')
        unique_together = (('field_def_type', 'field_def_id', 'order'),
                           ('field_def_type', 'field_def_id', 'group', 'value'))
    
    def clean(self):
        messages = {}
        
        try:
            self.field_def.field_instance().clean(self.value, None)
        except ValidationError as e:
            messages['value'] = e.messages
            
        if not isinstance(self.field_def, FieldDefinition):
            msg = _(u'This must be an instance of a `FieldDefinition`')
            messages['field_def'] = [msg]
        
        if messages:
            raise ValidationError(messages)
    
    def save(self, *args, **kwargs):
        save = super(FieldDefinitionChoice, self).save(*args, **kwargs)
        self.field_def.model_def.model_class(force_create=True)
        return save




from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db.models.deletion import ProtectedError

from ...models.model import ModelDefinition
from ...test.testcases import FieldDefinitionTestMixin
from ...tests.models.utils import BaseModelDefinitionTestCase

from .models import ForeignKeyDefinition, ManyToManyFieldDefinition


class RelatedFieldDefinitionTestMixin(FieldDefinitionTestMixin):
    
    def setUp(self):
        self.field_defintion_init_kwargs = {
            'to': ContentType.objects.get_for_model(ContentType),
            'null': True
        }
        super(RelatedFieldDefinitionTestMixin, self).setUp()

class ForeignKeyDefinitionTest(RelatedFieldDefinitionTestMixin,
                               BaseModelDefinitionTestCase):
    field_definition_cls = ForeignKeyDefinition
    
    def setUp(self):
        self.field_values = (
            ContentType.objects.get_for_model(ContentType),
            ContentType.objects.get_for_model(ModelDefinition),
        )
        super(ForeignKeyDefinitionTest, self).setUp()
    
    def prepare_default_value(self, value):
        return value.pk
    
    def test_simple_foreign_key_between_mutable_models(self):
        first_model_def = self.model_def
        second_model_def = ModelDefinition.objects.create(app_label='app',
                                                          object_name='SecondModel')
        FirstModel = first_model_def.model_class()
        SecondModel = second_model_def.model_class()
        ForeignKeyDefinition.objects.create(model_def=first_model_def,
                                            name='second', null=True,
                                            to=second_model_def.model_ct)
        
        second = SecondModel.objects.create()
        first = FirstModel.objects.create(second=second)
        
        ForeignKeyDefinition.objects.create(model_def=second_model_def,
                                            name='first', null=True,
                                            to=first_model_def.model_ct)
        
        second.first = first
        self.assertRaisesMessage(ValidationError,
                                 'Cannot save an obsolete model', second.save)
        
        second = SecondModel.objects.get()
        second.first = first
        second.save()
        
        second_model_def.delete()
        
    def test_recursive_relationship(self):
        fk = ForeignKeyDefinition.objects.create(model_def=self.model_def,
                                                 name='f1', null=True, blank=True,
                                                 to=self.model_def.model_ct)
        self.assertTrue(fk.is_recursive_relationship)
        
        Model = self.model_def.model_class()
        self.assertEqual(Model._meta.get_field('f1').rel.to, Model)

        obj1 = Model.objects.create()
        obj2 = Model.objects.create(f1=obj1)
        obj1.f1 = obj2
        obj1.save()
        
class ForeignKeyDefinitionOnDeleteTest(BaseModelDefinitionTestCase):
    
    def test_protect(self):
        ForeignKeyDefinition.objects.create(model_def=self.model_def,
                                            name='f1', null=True,
                                            to=self.model_def.model_ct,
                                            on_delete='PROTECT')
        
        Model = self.model_def.model_class()
        obj1 = Model.objects.create()
        Model.objects.create(f1=obj1)
        
        self.assertRaises(ProtectedError, obj1.delete)
        
    def test_set_null(self):
        fk = ForeignKeyDefinition(model_def=self.model_def,
                                  name='f1',
                                  to=self.model_def.model_ct,
                                  on_delete='SET_NULL')
        
        self.assertRaises(ValidationError, fk.clean)
        
        fk.null = True
        fk.save()
        Model = self.model_def.model_class()
        obj1 = Model.objects.create()
        obj2 = Model.objects.create(f1=obj1)
        obj1.delete()
        
        self.assertIsNone(Model.objects.get(pk=obj2.pk).f1)
        
    def test_set_default(self):
        Model = self.model_def.model_class()
        default = Model.objects.create().pk
        
        fk = ForeignKeyDefinition.objects.create(model_def=self.model_def,
                                                 name='f1', null=True,
                                                 to=self.model_def.model_ct,
                                                 on_delete='SET_DEFAULT')
        
        self.assertRaises(ValidationError, fk.clean)
        fk.default = default
        fk.save()
        
        obj1 = Model.objects.create()
        obj2 = Model.objects.create(f1=obj1)
        obj1.delete()
        
        self.assertEqual(Model.objects.get(pk=obj2.pk).f1.pk, default)
        
    def test_set_value(self):
        Model = self.model_def.model_class()
        default = Model.objects.create().pk
        
        fk = ForeignKeyDefinition.objects.create(model_def=self.model_def,
                                                 name='f1', null=True,
                                                 to=self.model_def.model_ct,
                                                 on_delete='SET_VALUE')
        
        fk.on_delete_set_value = default
        fk.save()
        
        obj1 = Model.objects.create()
        obj2 = Model.objects.create(f1=obj1)
        obj1.delete()
        
        self.assertEqual(Model.objects.get(pk=obj2.pk).f1.pk, default)
        
class ManyToManyFieldDefinitionTest(RelatedFieldDefinitionTestMixin,
                                    BaseModelDefinitionTestCase):
    field_definition_cls = ManyToManyFieldDefinition
    
    def setUp(self):
        self.field_values = (
            [ContentType.objects.get_for_model(ContentType)],
            [ContentType.objects.get_for_model(ModelDefinition),
             ContentType.objects.get_for_model(ContentType)]
        )
        super(ManyToManyFieldDefinitionTest, self).setUp()
    
    def get_field_value(self, instance, name='field'):
        value = super(RelatedFieldDefinitionTestMixin, self).get_field_value(instance, name)
        return list(value.all())
    
    def test_field_renaming(self):
        # TODO: investigate why this fails
        return
        value = self.field_values[0]
        Model = self.model_def.model_class()
        
        instance = Model.objects.create()
        instance.field = value
        
        self.field.name = 'renamed_field'
        self.field.save()
        
        instance = Model.objects.get()
        self.assertEqual(instance.renamed_field.all(), value)
        
        self.assertFalse(hasattr(Model, 'field'))
        
        instance = Model.objects.create()
        instance.renamed_field = value
    
    def test_field_default(self):
        # TODO: Investigate why this fails
        pass
    
    def test_model_save(self):
        # TODO: Investigate why this fails
        pass
    
    def test_field_unique(self):
        # TODO: Investigate why this fails
        pass
    
    def test_field_deletion(self):
        # TODO: Investigate why this fails
        pass
    
    def test_field_symmetrical(self):
        m2m = ManyToManyFieldDefinition(model_def=self.model_def, name='objs')
        ct_ct = ContentType.objects.get_for_model(ContentType)
        m2m.to = ct_ct
        
        with self.assertRaises(ValidationError):
            m2m.symmetrical = True
            m2m.clean()
            
        with self.assertRaises(ValidationError):
            m2m.symmetrical = False
            m2m.clean()
        
        m2m.to = self.model_def.model_ct
        
        # Make sure `symetrical=True` works
#        m2m.symmetrical = True
#        m2m.clean()
#        m2m.save()
#        
#        Model = self.model_def.model_class()
#        first_object = Model.objects.create()
#        second_object = Model.objects.create()
#        
#        first_object.objs.add(second_object)
#        self.assertIn(first_object, second_object.objs.all())
        
        # Makes sure non-symmetrical works
        m2m.symmetrical = False
        m2m.clean()
        m2m.save()
        
        Model = self.model_def.model_class()
        first_object = Model.objects.create()
        second_object = Model.objects.create()
        
        first_object.objs.add(second_object)
        self.assertNotIn(first_object, second_object.objs.all())
        
        first_object.objs.clear()




from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ..text.models import CharFieldDefinition
from ...db.fields.python import DirectoryPathField, RegExpStringField


path_help_text = _(u'The absolute filesystem path to a directory from which '
                   u'this field should get its choices.')
match_help_text = _(u'A regular expression used to filter filenames.')
recursive_help_text = _(u'Specifies whether all subdirectories of '
                        u'path should be included')

class FilePathFieldDefinition(CharFieldDefinition):
    
    path = DirectoryPathField(_(u'path'), max_length=100,
                              help_text=path_help_text)
    match = RegExpStringField(_(u'match'), max_length=100,
                              blank=True, null=True, help_text=match_help_text)
    recursive = fields.BooleanField(_(u'recursive'), default=False,
                                    help_text=recursive_help_text)
    
    class Meta:
        app_label = 'mutant'
        verbose_name = _(u'file path field')
        verbose_name_plural = _(u'file paths fields')
        defined_field_class = fields.FilePathField
        defined_field_options = ('path', 'match', 'recursive')
        defined_field_category = _(u'file')
